#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_firmware_upgrades_info
short_description: Information module for organizations _firmware _upgrades
description:
  - Information module for Organizations Firmware Upgrades Info.
  - Get all organizations _firmware _upgrades.
  - Get firmware upgrade information for an organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - Information module for Organizations Firmware Upgrades Info.
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - Information module for Organizations Firmware Upgrades Info.
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - Information module for Organizations Firmware Upgrades Info.
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - Information module for Organizations Firmware Upgrades Info.
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  status:
    description:
      - Information module for Organizations Firmware Upgrades Info.
      - Status query parameter. Optional parameter to filter the upgrade by status.
    elements: str
    type: list
  productTypes:
    description:
      - Information module for Organizations Firmware Upgrades Info.
      - ProductTypes query parameter. Optional parameter to filter the upgrade by
        product type.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationFirmwareUpgrades
    description: Complete reference of the getOrganizationFirmwareUpgrades API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-firmware-upgrades
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_firmware_upgrades,
  - Paths used are
    get /organizations/{organizationId}/firmware/upgrades,
"""

EXAMPLES = r"""
- name: Get all organizations _firmware _upgrades
  cisco.meraki.organizations_firmware_upgrades_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    status: []
    productTypes: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "upgradeId": "string",
        "upgradeBatchId": "string",
        "network": {
          "id": "string",
          "name": "string"
        },
        "status": "string",
        "time": "string",
        "completedAt": "string",
        "productTypes": "string",
        "toVersion": {
          "id": "string",
          "shortName": "string",
          "firmware": "string",
          "releaseType": "string",
          "releaseDate": "string"
        },
        "fromVersion": {
          "id": "string",
          "shortName": "string",
          "firmware": "string",
          "releaseType": "string",
          "releaseDate": "string"
        }
      }
    ]
"""



#!/usr/bin/python
# Copyright (c) 2026 Mariam Ahhttouche <mariam.ahhttouche@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: uv_python
short_description: Manage Python versions and installations using the C(uv) Python package manager
description:
  - Install, uninstall or upgrade Python versions managed by C(uv).
version_added: "13.0.0"
requirements:
  - C(uv) must be installed and available in E(PATH) and must be at least 0.8.0.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  version:
    description:
      - Python version to manage.
      - "Not all canonical Python versions are supported in this release. Valid version numbers consist of two or three dot-separated
        numeric components, with an optional 'pre-release' tag at the end such as V(3.12), V(3.12.3), V(3.15.0a5)."
      - Advanced uv selectors such as V(>=3.12,<3.13) or V(cpython@3.12) are not supported in this release.
      - When you specify only a major.minor version, make sure the number is enclosed in quotes so that it gets parsed correctly.
        Note that in case only a major.minor version are specified behavior depends on the O(state) parameter.
    type: str
    required: true
  state:
    description:
      - Desired state of the specified Python version.
      - "V(present) ensures the specified version is installed. If you specify a full patch version (for example O(version=3.12.3)),
        that exact version is be installed if not already present. If you only specify a minor version (for example V(3.12)),
        the latest available patch version for that minor release is installed only if no patch version for that minor release
        is currently installed (including patch versions not managed by C(uv)). RV(python_versions) and RV(python_paths)
        lengths are always equal to one for this state."
      - "V(absent) ensures the specified version is removed. If you specify a full patch version, only that exact patch version
        is removed. If you only specify a minor version (for example V(3.12)), all installed patch versions for that minor
        release are removed. If you specify a version that is not installed, no changes are made. RV(python_versions) and
        RV(python_paths) lengths can be higher or equal to one in this state."
      - V(latest) ensures the latest available patch version for the specified version is installed. If you only specify
        a minor version (for example V(3.12)), the latest available patch version for that minor release is always installed.
        If another patch version is already installed but is not the latest, the latest patch version is installed. The latest
        patch version installed depends on the C(uv) version, since available Python versions are frozen per C(uv) release.
        RV(python_versions) and RV(python_paths) lengths are always equal to one in this state. This state does not use C(uv python upgrade).
    type: str
    choices: [present, absent, latest]
    default: present
seealso:
  - name: C(uv) documentation
    description: Python versions management with C(uv).
    link: https://docs.astral.sh/uv/concepts/python-versions/
  - name: C(uv) CLI documentation
    description: C(uv) CLI reference guide.
    link: https://docs.astral.sh/uv/reference/cli/#uv-python
author: Mariam Ahhttouche (@mriamah)
"""

EXAMPLES = r"""
- name: Install Python 3.14
  community.general.uv_python:
    version: "3.14"

- name: Upgrade Python 3.14
  community.general.uv_python:
    version: "3.14"
    state: latest

- name: Remove Python 3.13.5
  community.general.uv_python:
    version: 3.13.5
    state: absent
"""

RETURN = r"""
python_versions:
  description: List of Python versions changed.
  returned: success
  type: list
  elements: str
  sample:
    - "3.13.5"
python_paths:
  description: List of installation paths of Python versions changed.
  returned: success
  type: list
  elements: str
  sample:
    - "/root/.local/share/uv/python/cpython-3.13.5-linux-x86_64-gnu/bin/python3.13"
stdout:
  description: Stdout of the executed command.
  returned: success
  type: str
  sample: ""
stderr:
  description: Stderr of the executed command.
  returned: success
  type: str
  sample: ""
rc:
  description: Return code of the executed command.
  returned: success
  type: int
  sample: 0
"""

import json
import re
from pathlib import Path

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.compat.version import LooseVersion, StrictVersion

MINIMUM_UV_VERSION = "0.8.0"


class UV:
    """
    Module for managing Python versions and installations using "uv python" command
    """

    def __init__(self, module: AnsibleModule) -> None:
        self.module = module
        self.bin_path = self.module.get_bin_path("uv", required=True)
        self._ensure_min_uv_version()
        try:
            self.python_version = StrictVersion(module.params["version"])
            self.python_version_str = str(self.python_version)
        except (ValueError, AttributeError):
            self.module.fail_json(
                msg=(
                    "Unsupported version format. Valid version numbers consist of two or three dot-separated numeric components"
                    " with an optional 'pre-release' tag on the end (for example 3.12, 3.12.3, 3.15.0a5) are supported in this release."
                )
            )

    def _ensure_min_uv_version(self) -> None:
        cmd = [self.bin_path, "--version", "--color", "never"]
        dummy_rc, out, dummy_err = self.module.run_command(cmd, check_rc=True)
        try:
            detected = re.search(r"\b\d+(?:\.\d+)+\b", out).group()  # type: ignore[union-attr]
            if LooseVersion(detected) < LooseVersion(MINIMUM_UV_VERSION):
                self.module.fail_json(
                    msg=f"uv_python module requires uv >= {MINIMUM_UV_VERSION}",
                    detected_version=detected,
                    required_version=MINIMUM_UV_VERSION,
                )
        except AttributeError:
            self.module.warn("Could not get installed uv version, skipping uv version check")

    def install_python(self) -> tuple[bool, str, str, int, list[str], list[str]]:
        """
        Runs command 'uv python install X.Y.Z' which installs specified python version.
        If patch version is not specified uv installs latest available patch version.
        Returns:
          - boolean to indicate if method changed state
          - command's stdout
          - command's stderr
          - command's return code
          - list of installed versions
          - list of installation paths for each installed version
        """
        find_rc, existing_version, dummy_err = self._find_python("--show-version")
        if find_rc == 0:
            dummy_rc, version_path, dummy_err = self._find_python()
            return False, "", "", 0, [existing_version], [version_path]
        if self.module.check_mode:
            latest_version, dummy_path = self._get_latest_patch_release("--managed-python")
            # when uv does not find any available patch version the install command will fail
            if not latest_version:
                self.module.fail_json(msg=(f"Version {self.python_version_str} is not available."))
            return True, "", "", 0, [latest_version], [""]
        rc, out, err = self._exec(self.python_version_str, "install", check_rc=True)
        latest_version, path = self._get_latest_patch_release("--only-installed", "--managed-python")
        return True, out, err, rc, [latest_version], [path]

    def uninstall_python(self) -> tuple[bool, str, str, int, list, list]:
        """
        Runs command 'uv python uninstall X.Y.Z' which removes specified python version from environment.
        If patch version is not specified all correspending installed patch versions are removed.
        Returns:
          tuple [bool, str, str, int, list, list]
          - boolean to indicate if method changed state
          - command's stdout
          - command's stderr
          - command's return code
          - list of uninstalled versions
          - list of previous installation paths for each uninstalled version
        """
        installed_versions, install_paths = self._get_installed_versions("--managed-python")
        if not installed_versions:
            return False, "", "", 0, [], []
        if self.module.check_mode:
            return True, "", "", 0, installed_versions, install_paths
        rc, out, err = self._exec(self.python_version_str, "uninstall", check_rc=True)
        return True, out, err, rc, installed_versions, install_paths

    def upgrade_python(self) -> tuple[bool, str, str, int, list, list]:
        """
        Runs command 'uv python install X.Y.Z' with latest patch version available.
        Returns:
          - boolean to indicate if method changed state
          - command's stdout
          - command's stderr
          - command's return code
          - list of installed versions
          - list of installation paths for each installed version
        """
        rc, installed_version_str, dummy_err = self._find_python("--show-version")
        installed_version = self._parse_version(installed_version_str)
        latest_version_str, dummy_path = self._get_latest_patch_release("--managed-python")
        if not latest_version_str:
            self.module.fail_json(msg=f"Version {self.python_version_str} is not available.")
        if rc == 0 and installed_version >= StrictVersion(latest_version_str):
            dummy_rc, install_path, dummy_err = self._find_python()
            return False, "", "", rc, [installed_version_str], [install_path]
        if self.module.check_mode:
            return True, "", "", 0, [latest_version_str], []
        # it's possible to have latest version already installed but not used as default
        # so in this case 'uv python install' will set latest version as default
        rc, out, err = self._exec(latest_version_str, "install", check_rc=True)
        latest_version_str, latest_path = self._get_latest_patch_release("--only-installed", "--managed-python")
        return True, out, err, rc, [latest_version_str], [latest_path]

    def _exec(self, python_version: str, command: str, *args, check_rc: bool = False) -> tuple[int, str, str]:
        """
        Execute a uv python subcommand.
        Args:
          python_version: Python version specifier (e.g. "3.12", "3.12.3").
          command (str): uv python subcommand (e.g. "install", "uninstall", "find").
          *args: Additional positional arguments passed to the command.
          check_rc (bool): Whether to fail if the command exits with non-zero return code.
        """
        cmd = [
            self.bin_path,
            "python",
            command,
            python_version,
            "--color",
            "never",
            *args,
        ]
        rc, out, err = self.module.run_command(cmd, check_rc=check_rc)
        return rc, out, err

    def _find_python(self, *args, check_rc: bool = False) -> tuple[int, str, str]:
        """
        Runs command 'uv python find' which returns path of installed patch releases for a given python version.
        If multiple patch versions are installed, "uv python find" returns the one used by default
        if inside a virtualenv otherwise it returns latest installed patch version.
        Args:
          *args: Additional positional arguments passed to _exec.
          check_rc (bool): Whether to fail if the command exits with non-zero return code.
        """
        rc, out, err = self._exec(self.python_version_str, "find", *args, check_rc=check_rc)
        if rc == 0:
            out = out.strip()
        return rc, out, err

    def _list_python(self, *args, check_rc: bool = False) -> tuple[int, list, str]:
        """
        Runs command 'uv python list' (which returns list of installed patch releases for a given python version).
        Official documentation https://docs.astral.sh/uv/reference/cli/#uv-python-list
        Args:
          *args: Additional positional arguments passed to _exec.
          check_rc (bool): Whether to fail if the command exits with non-zero return code.
        """
        rc, out, err = self._exec(
            self.python_version_str,
            "list",
            "--output-format",
            "json",
            *args,
            check_rc=check_rc,
        )
        pythons_installed = []
        try:
            pythons_installed = json.loads(out)
            # convert path to absolute path since in some recent uv releases "uv python list" returns relative install paths instead of absolute paths
            for result in pythons_installed:
                path = result.get("path", "")
                if path and not Path(path).is_absolute():
                    result["path"] = str(Path(path).resolve())
        except json.decoder.JSONDecodeError:
            self.module.debug("No Python installation found.")
        return rc, pythons_installed, err

    def _get_latest_patch_release(self, *args) -> tuple[str, str]:
        """
        Returns latest available patch release for a given python version.
        Args:
          *args: Additional positional arguments passed to _list_python.
        Returns:
            - latest found patch version in format X.Y.Z
            - installation path of latest patch version if version exists
        """
        latest_version = path = ""
        # 'uv python list' returns versions in descending order but we sort them just in case future uv behavior changes
        dummy_rc, results, dummy_err = self._list_python(*args)
        valid_results = self._filter_valid_versions(results)
        if valid_results:
            version = max(valid_results, key=lambda result: result["parsed_version"])
            latest_version = version.get("version", "")
            path = version.get("path", "")
        return latest_version, path

    def _get_installed_versions(self, *args) -> tuple[list, list]:
        """
        Returns installed patch releases for a given python version.
        Args:
          *args: Additional positional arguments passed to _list_python.
        Returns:
            - list of latest found patch versions
            - list of installation paths of installed versions
        """
        dummy_rc, results, dummy_err = self._list_python("--only-installed", *args)
        if results:
            return [result.get("version") for result in results], [result.get("path") for result in results]
        return [], []

    def _filter_valid_versions(self, results: list):
        valid_results = []
        for result in results:
            version = result.get("version", "")
            try:
                result["parsed_version"] = StrictVersion(version)
                valid_results.append(result)
            except ValueError:
                self.module.debug(f"Filtering out version {version!r} since it's not supported by uv_python module.")
        return valid_results

    @staticmethod
    def _parse_version(version_str: str):
        try:
            return StrictVersion(version_str)
        except ValueError:
            return StrictVersion("0")


def main():
    module = AnsibleModule(
        argument_spec=dict(
            version=dict(type="str", required=True),
            state=dict(type="str", default="present", choices=["present", "absent", "latest"]),
        ),
        supports_check_mode=True,
    )

    result = dict(
        changed=False,
        stdout="",
        stderr="",
        rc=0,
        python_versions=[],
        python_paths=[],
        failed=False,
    )
    state = module.params["state"]
    exec_result = {}
    uv = UV(module)

    if state == "present":
        exec_result = dict(
            zip(
                [
                    "changed",
                    "stdout",
                    "stderr",
                    "rc",
                    "python_versions",
                    "python_paths",
                ],
                uv.install_python(),
            )
        )
    elif state == "absent":
        exec_result = dict(
            zip(
                [
                    "changed",
                    "stdout",
                    "stderr",
                    "rc",
                    "python_versions",
                    "python_paths",
                ],
                uv.uninstall_python(),
            )
        )
    elif state == "latest":
        exec_result = dict(
            zip(
                [
                    "changed",
                    "stdout",
                    "stderr",
                    "rc",
                    "python_versions",
                    "python_paths",
                ],
                uv.upgrade_python(),
            )
        )

    result.update(exec_result)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2014, Dan Keder <dan.keder@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: seport
short_description: Manages SELinux network port type definitions
description:
  - Manages SELinux network port type definitions.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  ports:
    description:
      - Ports or port ranges.
      - Can be a list (since 2.6) or comma separated string.
    type: list
    elements: str
    required: true
  proto:
    description:
      - Protocol for the specified port.
      - Support for V(dccp) and V(sctp) has been added in community.general 12.4.0.
    type: str
    required: true
    choices: [tcp, udp, dccp, sctp]
  setype:
    description:
      - SELinux type for the specified port.
    type: str
    required: true
  state:
    description:
      - Desired boolean value.
    type: str
    choices: [absent, present]
    default: present
  reload:
    description:
      - Reload SELinux policy after commit.
    type: bool
    default: true
  ignore_selinux_state:
    description:
      - Run independent of selinux runtime state.
    type: bool
    default: false
  local:
    description:
      - Work with local modifications only.
    type: bool
    default: false
    version_added: 5.6.0
notes:
  - The changes are persistent across reboots.
  - Not tested on any Debian based system.
requirements:
  - libselinux-python
  - policycoreutils-python
author:
  - Dan Keder (@dankeder)
"""

EXAMPLES = r"""
- name: Allow Apache to listen on tcp port 8888
  community.general.seport:
    ports: 8888
    proto: tcp
    setype: http_port_t
    state: present

- name: Allow sshd to listen on tcp port 8991
  community.general.seport:
    ports: 8991
    proto: tcp
    setype: ssh_port_t
    state: present

- name: Allow memcached to listen on tcp ports 10000-10100 and 10112
  community.general.seport:
    ports: 10000-10100,10112
    proto: tcp
    setype: memcache_port_t
    state: present

- name: Allow memcached to listen on tcp ports 10000-10100 and 10112
  community.general.seport:
    ports:
      - 10000-10100
      - 10112
    proto: tcp
    setype: memcache_port_t
    state: present

- name: Remove tcp port 22 local modification if exists
  community.general.seport:
    ports: 22
    protocol: tcp
    setype: ssh_port_t
    state: absent
    local: true
"""

import traceback

SELINUX_IMP_ERR = None
try:
    import selinux

    HAVE_SELINUX = True
except ImportError:
    SELINUX_IMP_ERR = traceback.format_exc()
    HAVE_SELINUX = False

SEOBJECT_IMP_ERR = None
try:
    import seobject

    HAVE_SEOBJECT = True
except ImportError:
    SEOBJECT_IMP_ERR = traceback.format_exc()
    HAVE_SEOBJECT = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib


def get_runtime_status(ignore_selinux_state=False):
    return ignore_selinux_state or selinux.is_selinux_enabled()


def semanage_port_get_ports(seport, setype, proto, local):
    """Get the list of ports that have the specified type definition.

    :param community.general.seport: Instance of seobject.portRecords

    :type setype: str
    :param setype: SELinux type.

    :type proto: str
    :param proto: Protocol ('tcp', 'udp', 'dccp', 'sctp')

    :rtype: list
    :return: List of ports that have the specified SELinux type.
    """
    records = seport.get_all_by_type(locallist=local)
    if (setype, proto) in records:
        return records[(setype, proto)]
    else:
        return []


def _parse_port_range(port):
    """Return (low, high) integers for a port or port range string."""
    parts = str(port).split("-", 1)
    low = int(parts[0])
    high = int(parts[1]) if len(parts) == 2 else low
    return low, high


def _port_is_covered(port, existing_ports):
    """Return True if port (or range) is fully covered by an existing port entry."""
    req_low, req_high = _parse_port_range(port)
    for entry in existing_ports:
        entry_low, entry_high = _parse_port_range(entry)
        if entry_low <= req_low and req_high <= entry_high:
            return True
    return False


def semanage_port_get_type(seport, port, proto):
    """Get the SELinux type of the specified port.

    :param community.general.seport: Instance of seobject.portRecords

    :type port: str
    :param port: Port or port range (example: "8080", "8080-9090")

    :type proto: str
    :param proto: Protocol ('tcp', 'udp', 'dccp', 'sctp')

    :rtype: tuple
    :return: Tuple containing the SELinux type and MLS/MCS level, or None if not found.
    """
    if isinstance(port, str):
        ports = port.split("-", 1)
        if len(ports) == 1:
            ports.extend(ports)
    else:
        ports = (port, port)

    key = (int(ports[0]), int(ports[1]), proto)

    records = seport.get_all()
    return records.get(key)


def semanage_port_add(module, ports, proto, setype, do_reload, serange="s0", sestore="", local=False):
    """Add SELinux port type definition to the policy.

    :type module: AnsibleModule
    :param module: Ansible module

    :type ports: list
    :param ports: List of ports and port ranges to add (e.g. ["8080", "8080-9090"])

    :type proto: str
    :param proto: Protocol ('tcp', 'udp', 'dccp', 'sctp')

    :type setype: str
    :param setype: SELinux type

    :type do_reload: bool
    :param do_reload: Whether to reload SELinux policy after commit

    :type serange: str
    :param serange: SELinux MLS/MCS range (defaults to 's0')

    :type sestore: str
    :param sestore: SELinux store

    :rtype: bool
    :return: True if the policy was changed, otherwise False
    """
    change = False
    try:
        seport = seobject.portRecords(sestore)
        seport.set_reload(do_reload)
        ports_by_type = semanage_port_get_ports(seport, setype, proto, local)
        for port in ports:
            if _port_is_covered(port, ports_by_type):
                continue

            change = True
            if module.check_mode:
                continue
            port_type = semanage_port_get_type(seport, port, proto)
            if port_type is None:
                seport.add(port, proto, serange, setype)
            else:
                seport.modify(port, proto, serange, setype)

    except FileNotFoundError as e:
        module.fail_json(
            msg=f"Failed to modify SELinux port policy, possibly due to a port overlap with an existing range: {e}\n",
            exception=traceback.format_exc(),
        )
    except (ValueError, OSError, KeyError, RuntimeError) as e:
        module.fail_json(msg=f"{e.__class__.__name__}: {e}\n", exception=traceback.format_exc())

    return change


def semanage_port_del(module, ports, proto, setype, do_reload, sestore="", local=False):
    """Delete SELinux port type definition from the policy.

    :type module: AnsibleModule
    :param module: Ansible module

    :type ports: list
    :param ports: List of ports and port ranges to delete (e.g. ["8080", "8080-9090"])

    :type proto: str
    :param proto: Protocol ('tcp', 'udp', 'dccp', 'sctp')

    :type setype: str
    :param setype: SELinux type.

    :type do_reload: bool
    :param do_reload: Whether to reload SELinux policy after commit

    :type sestore: str
    :param sestore: SELinux store

    :rtype: bool
    :return: True if the policy was changed, otherwise False
    """
    change = False
    try:
        seport = seobject.portRecords(sestore)
        seport.set_reload(do_reload)
        ports_by_type = semanage_port_get_ports(seport, setype, proto, local)
        for port in ports:
            if port in ports_by_type:
                change = True
                if not module.check_mode:
                    seport.delete(port, proto)

    except FileNotFoundError as e:
        module.fail_json(
            msg=f"Failed to modify SELinux port policy, possibly due to a port overlap with an existing range: {e}\n",
            exception=traceback.format_exc(),
        )
    except (ValueError, OSError, KeyError, RuntimeError) as e:
        module.fail_json(msg=f"{e.__class__.__name__}: {e}\n", exception=traceback.format_exc())

    return change


def main():
    module = AnsibleModule(
        argument_spec=dict(
            ignore_selinux_state=dict(type="bool", default=False),
            ports=dict(type="list", elements="str", required=True),
            proto=dict(type="str", required=True, choices=["tcp", "udp", "dccp", "sctp"]),
            setype=dict(type="str", required=True),
            state=dict(type="str", default="present", choices=["absent", "present"]),
            reload=dict(type="bool", default=True),
            local=dict(type="bool", default=False),
        ),
        supports_check_mode=True,
    )

    if not HAVE_SELINUX:
        module.fail_json(msg=missing_required_lib("libselinux-python"), exception=SELINUX_IMP_ERR)

    if not HAVE_SEOBJECT:
        module.fail_json(msg=missing_required_lib("policycoreutils-python"), exception=SEOBJECT_IMP_ERR)

    ignore_selinux_state = module.params["ignore_selinux_state"]

    if not get_runtime_status(ignore_selinux_state):
        module.fail_json(msg="SELinux is disabled on this host.")

    ports = module.params["ports"]
    proto = module.params["proto"]
    setype = module.params["setype"]
    state = module.params["state"]
    do_reload = module.params["reload"]
    local = module.params["local"]

    result = {
        "ports": ports,
        "proto": proto,
        "setype": setype,
        "state": state,
    }

    if state == "present":
        result["changed"] = semanage_port_add(module, ports, proto, setype, do_reload, local=local)
    elif state == "absent":
        result["changed"] = semanage_port_del(module, ports, proto, setype, do_reload, local=local)
    else:
        module.fail_json(msg=f'Invalid value of argument "state": {state}')

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2016-2017 Hewlett Packard Enterprise Development LP
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: oneview_fc_network
short_description: Manage OneView Fibre Channel Network resources
description:
  - Provides an interface to manage Fibre Channel Network resources. Can create, update, and delete.
requirements:
  - "hpOneView >= 4.0.0"
author: "Felipe Bulsoni (@fgbulsoni)"
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  state:
    description:
      - Indicates the desired state for the Fibre Channel Network resource.
      - V(present) ensures data properties are compliant with OneView.
      - V(absent) removes the resource from OneView, if it exists.
    type: str
    choices: ['present', 'absent']
    required: true
  data:
    description:
      - List with the Fibre Channel Network properties.
    type: dict
    required: true

extends_documentation_fragment:
  - community.general._oneview
  - community.general._oneview.validateetag
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Ensure that the Fibre Channel Network is present using the default configuration
  community.general.oneview_fc_network:
    config: "{{ config_file_path }}"
    state: present
    data:
      name: 'New FC Network'

- name: Ensure that the Fibre Channel Network is present with fabricType 'DirectAttach'
  community.general.oneview_fc_network:
    config: "{{ config_file_path }}"
    state: present
    data:
      name: 'New FC Network'
      fabricType: 'DirectAttach'

- name: Ensure that the Fibre Channel Network is present and is inserted in the desired scopes
  community.general.oneview_fc_network:
    config: "{{ config_file_path }}"
    state: present
    data:
      name: 'New FC Network'
      scopeUris:
        - '/rest/scopes/00SC123456'
        - '/rest/scopes/01SC123456'

- name: Ensure that the Fibre Channel Network is absent
  community.general.oneview_fc_network:
    config: "{{ config_file_path }}"
    state: absent
    data:
      name: 'New FC Network'
"""

RETURN = r"""
fc_network:
  description: Has the facts about the managed OneView FC Network.
  returned: On O(state=present). Can be null.
  type: dict
"""

from ansible_collections.community.general.plugins.module_utils._oneview import OneViewModuleBase


class FcNetworkModule(OneViewModuleBase):
    MSG_CREATED = "FC Network created successfully."
    MSG_UPDATED = "FC Network updated successfully."
    MSG_DELETED = "FC Network deleted successfully."
    MSG_ALREADY_PRESENT = "FC Network is already present."
    MSG_ALREADY_ABSENT = "FC Network is already absent."
    RESOURCE_FACT_NAME = "fc_network"

    def __init__(self):
        additional_arg_spec = dict(
            data=dict(required=True, type="dict"), state=dict(required=True, choices=["present", "absent"])
        )

        super().__init__(additional_arg_spec=additional_arg_spec, validate_etag_support=True)

        self.resource_client = self.oneview_client.fc_networks

    def execute_module(self):
        resource = self.get_by_name(self.data["name"])

        if self.state == "present":
            return self._present(resource)
        else:
            return self.resource_absent(resource)

    def _present(self, resource):
        scope_uris = self.data.pop("scopeUris", None)
        result = self.resource_present(resource, self.RESOURCE_FACT_NAME)
        if scope_uris is not None:
            result = self.resource_scopes_set(result, "fc_network", scope_uris)
        return result


def main():
    FcNetworkModule().run()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2018, Milan Ilic <milani@nordeus.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Make coding more python3-ish
from __future__ import annotations

DOCUMENTATION = r"""
module: one_image_info
short_description: Gather information on OpenNebula images
description:
  - Gather information on OpenNebula images.
requirements:
  - pyone
extends_documentation_fragment:
  - community.general._opennebula
  - community.general._attributes
  - community.general._attributes.info_module
options:
  ids:
    description:
      - A list of images IDs whose facts you want to gather.
      - Module can use integers too.
    aliases: ['id']
    type: list
    elements: str
  name:
    description:
      - A O(name) of the image whose facts is gathered.
      - If the O(name) begins with V(~) the O(name) is used as regex pattern, which restricts the list of images (whose facts
        is returned) whose names match specified regex.
      - Also, if the O(name) begins with V(~*) case-insensitive matching is performed.
      - See examples for more details.
    type: str
author:
  - "Milan Ilic (@ilicmilan)"
  - "Jan Meerkamp (@meerkampdvv)"
"""

EXAMPLES = r"""
- name: Gather facts about all images
  community.general.one_image_info:
  register: result

- name: Print all images facts
  ansible.builtin.debug:
    msg: result

- name: Gather facts about an image using ID
  community.general.one_image_info:
    ids: 123

- name: Gather facts about an image using list of ID
  community.general.one_image_info:
    ids:
      - 123
      - 456
      - 789
      - 0

- name: Gather facts about an image using the name
  community.general.one_image_info:
    name: 'foo-image'
  register: foo_image

- name: Gather facts about all IMAGEs whose name matches regex 'app-image-.*'
  community.general.one_image_info:
    name: '~app-image-.*'
  register: app_images

- name: Gather facts about all IMAGEs whose name matches regex 'foo-image-.*' ignoring cases
  community.general.one_image_info:
    name: '~*foo-image-.*'
  register: foo_images
"""

RETURN = r"""
images:
  description: A list of images info.
  type: list
  elements: dict
  returned: success
  contains:
    id:
      description: The image's ID.
      type: int
      sample: 153
    name:
      description: The image's name.
      type: str
      sample: app1
    group_id:
      description: The image's group ID.
      type: int
      sample: 1
    group_name:
      description: The image's group name.
      type: str
      sample: one-users
    owner_id:
      description: The image's owner ID.
      type: int
      sample: 143
    owner_name:
      description: The image's owner name.
      type: str
      sample: ansible-test
    state:
      description: The image's state.
      type: str
      sample: READY
    used:
      description: The image's usage status.
      type: bool
      sample: true
    running_vms:
      description: The image's count of running vms that use this image.
      type: int
      sample: 7
    permissions:
      description: The image's permissions.
      type: dict
      version_added: 9.5.0
      contains:
        owner_u:
          description: The image's owner USAGE permissions.
          type: str
          sample: 1
        owner_m:
          description: The image's owner MANAGE permissions.
          type: str
          sample: 0
        owner_a:
          description: The image's owner ADMIN permissions.
          type: str
          sample: 0
        group_u:
          description: The image's group USAGE permissions.
          type: str
          sample: 0
        group_m:
          description: The image's group MANAGE permissions.
          type: str
          sample: 0
        group_a:
          description: The image's group ADMIN permissions.
          type: str
          sample: 0
        other_u:
          description: The image's other users USAGE permissions.
          type: str
          sample: 0
        other_m:
          description: The image's other users MANAGE permissions.
          type: str
          sample: 0
        other_a:
          description: The image's other users ADMIN permissions.
          type: str
          sample: 0
      sample:
        owner_u: 1
        owner_m: 0
        owner_a: 0
        group_u: 0
        group_m: 0
        group_a: 0
        other_u: 0
        other_m: 0
        other_a: 0
    type:
      description: The image's type.
      type: int
      sample: 0
      version_added: 9.5.0
    disk_type:
      description: The image's format type.
      type: int
      sample: 0
      version_added: 9.5.0
    persistent:
      description: The image's persistence status (1 means true, 0 means false).
      type: int
      sample: 1
      version_added: 9.5.0
    source:
      description: The image's source.
      type: str
      sample: /var/lib/one//datastores/100/somerandomstringxd
      version_added: 9.5.0
    path:
      description: The image's filesystem path.
      type: str
      sample: /var/tmp/hello.qcow2
      version_added: 9.5.0
    fstype:
      description: The image's filesystem type.
      type: str
      sample: ext4
      version_added: 9.5.0
    size:
      description: The image's size in MegaBytes.
      type: int
      sample: 10000
      version_added: 9.5.0
    cloning_ops:
      description: The image's cloning operations per second.
      type: int
      sample: 0
      version_added: 9.5.0
    cloning_id:
      description: The image's cloning ID.
      type: int
      sample: -1
      version_added: 9.5.0
    target_snapshot:
      description: The image's target snapshot.
      type: int
      sample: 1
      version_added: 9.5.0
    datastore_id:
      description: The image's datastore ID.
      type: int
      sample: 100
      version_added: 9.5.0
    datastore:
      description: The image's datastore name.
      type: int
      sample: image_datastore
      version_added: 9.5.0
    vms:
      description: The image's list of VM ID's.
      type: list
      elements: int
      version_added: 9.5.0
      sample:
        - 1
        - 2
        - 3
    clones:
      description: The image's list of clones ID's.
      type: list
      elements: int
      version_added: 9.5.0
      sample:
        - 1
        - 2
        - 3
    app_clones:
      description: The image's list of app_clones ID's.
      type: list
      elements: int
      version_added: 9.5.0
      sample:
        - 1
        - 2
        - 3
    snapshots:
      description: The image's list of snapshots.
      type: list
      version_added: 9.5.0
      sample:
        - date: 123123
          parent: 1
          size: 10228
          allow_orphans: 1
          children: 0
          active: 1
          name: SampleName
"""

import re

from ansible_collections.community.general.plugins.module_utils._opennebula import OpenNebulaModule

IMAGE_STATES = [
    "INIT",
    "READY",
    "USED",
    "DISABLED",
    "LOCKED",
    "ERROR",
    "CLONE",
    "DELETE",
    "USED_PERS",
    "LOCKED_USED",
    "LOCKED_USED_PERS",
]


class ImageInfoModule(OpenNebulaModule):
    def __init__(self):
        argument_spec = dict(
            ids=dict(type="list", aliases=["id"], elements="str"),
            name=dict(type="str"),
        )
        mutually_exclusive = [
            ["ids", "name"],
        ]

        OpenNebulaModule.__init__(self, argument_spec, supports_check_mode=True, mutually_exclusive=mutually_exclusive)

    def run(self, one, module, result):
        params = module.params
        ids = params.get("ids")
        name = params.get("name")

        if ids:
            images = self.get_images_by_ids(ids)
        elif name:
            images = self.get_images_by_name(name)
        else:
            images = self.get_all_images().IMAGE

        self.result = {"images": [self.get_image_info(image) for image in images]}

        self.exit()

    def get_all_images(self):
        pool = self.one.imagepool.info(-2, -1, -1, -1)
        # Filter -2 means fetch all images user can Use

        return pool

    def get_images_by_ids(self, ids):
        images = []
        pool = self.get_all_images()

        for image in pool.IMAGE:
            if str(image.ID) in ids:
                images.append(image)
                ids.remove(str(image.ID))
                if len(ids) == 0:
                    break

        if len(ids) > 0:
            self.module.fail_json(msg=f"There is no IMAGE(s) with id(s)={', '.join(str(image_id) for image_id in ids)}")

        return images

    def get_images_by_name(self, name_pattern):
        images = []
        pattern = None

        pool = self.get_all_images()

        if name_pattern.startswith("~"):
            if name_pattern[1] == "*":
                pattern = re.compile(name_pattern[2:], re.IGNORECASE)
            else:
                pattern = re.compile(name_pattern[1:])

        for image in pool.IMAGE:
            if pattern is not None:
                if pattern.match(image.NAME):
                    images.append(image)
            elif name_pattern == image.NAME:
                images.append(image)
                break

        # if the specific name is indicated
        if pattern is None and len(images) == 0:
            self.module.fail_json(msg=f"There is no IMAGE with name={name_pattern}")

        return images


def main():
    ImageInfoModule().run_module()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: vertica_schema
short_description: Adds or removes Vertica database schema and roles
description:
  - Adds or removes Vertica database schema and, optionally, roles with schema access privileges.
  - A schema is not removed until all the objects have been dropped.
  - In such a situation, if the module tries to remove the schema it fails and only remove roles created for the schema if
    they have no dependencies.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  schema:
    description:
      - Name of the schema to add or remove.
    required: true
    aliases: ['name']
    type: str
  usage_roles:
    description:
      - Comma separated list of roles to create and grant usage access to the schema.
    aliases: ['usage_role']
    type: str
  create_roles:
    description:
      - Comma separated list of roles to create and grant usage and create access to the schema.
    aliases: ['create_role']
    type: str
  owner:
    description:
      - Name of the user to set as owner of the schema.
    type: str
  state:
    description:
      - Whether to create V(present), or drop V(absent) a schema.
    default: present
    choices: ['present', 'absent']
    type: str
  db:
    description:
      - Name of the Vertica database.
    type: str
  cluster:
    description:
      - Name of the Vertica cluster.
    default: localhost
    type: str
  port:
    description:
      - Vertica cluster port to connect to.
    default: '5433'
    type: str
  login_user:
    description:
      - The username used to authenticate with.
    default: dbadmin
    type: str
  login_password:
    description:
      - The password used to authenticate with.
    type: str
notes:
  - The default authentication assumes that you are either logging in as or sudo'ing to the C(dbadmin) account on the host.
  - This module uses C(pyodbc), a Python ODBC database adapter. You must ensure that C(unixODBC) and C(pyodbc) is installed
    on the host and properly configured.
  - Configuring C(unixODBC) for Vertica requires C(Driver = /opt/vertica/lib64/libverticaodbc.so) to be added to the C(Vertica)
    section of either C(/etc/odbcinst.ini) or C($HOME/.odbcinst.ini) and both C(ErrorMessagesPath = /opt/vertica/lib64) and
    C(DriverManagerEncoding = UTF-16) to be added to the C(Driver) section of either C(/etc/vertica.ini) or C($HOME/.vertica.ini).
requirements: ['unixODBC', 'pyodbc']
author: "Dariusz Owczarek (@dareko)"
"""

EXAMPLES = r"""
- name: Creating a new vertica schema
  community.general.vertica_schema: name=schema_name db=db_name state=present

- name: Creating a new schema with specific schema owner
  community.general.vertica_schema: name=schema_name owner=dbowner db=db_name state=present

- name: Creating a new schema with roles
  community.general.vertica_schema: name=schema_name create_roles=schema_name_all usage_roles=schema_name_ro,schema_name_rw
    db=db_name state=present
"""
import traceback

PYODBC_IMP_ERR = None
try:
    import pyodbc
except ImportError:
    PYODBC_IMP_ERR = traceback.format_exc()
    pyodbc_found = False
else:
    pyodbc_found = True

from ansible.module_utils.basic import AnsibleModule, missing_required_lib


class NotSupportedError(Exception):
    pass


class CannotDropError(Exception):
    pass


# module specific functions


def get_schema_facts(cursor, schema=""):
    facts = {}
    cursor.execute(
        """
        select schema_name, schema_owner, create_time
        from schemata
        where not is_system_schema and schema_name not in ('public', 'TxtIndex')
        and (? = '' or schema_name ilike ?)
    """,
        schema,
        schema,
    )
    while True:
        rows = cursor.fetchmany(100)
        if not rows:
            break
        for row in rows:
            facts[row.schema_name.lower()] = {
                "name": row.schema_name,
                "owner": row.schema_owner,
                "create_time": str(row.create_time),
                "usage_roles": [],
                "create_roles": [],
            }
    cursor.execute(
        """
        select g.object_name as schema_name, r.name as role_name,
        lower(g.privileges_description) privileges_description
        from roles r join grants g
        on g.grantee_id = r.role_id and g.object_type='SCHEMA'
        and g.privileges_description like '%USAGE%'
        and g.grantee not in ('public', 'dbadmin')
        and (? = '' or g.object_name ilike ?)
    """,
        schema,
        schema,
    )
    while True:
        rows = cursor.fetchmany(100)
        if not rows:
            break
        for row in rows:
            schema_key = row.schema_name.lower()
            if "create" in row.privileges_description:
                facts[schema_key]["create_roles"].append(row.role_name)
            else:
                facts[schema_key]["usage_roles"].append(row.role_name)
    return facts


def update_roles(schema_facts, cursor, schema, existing, required, create_existing, create_required):
    for role in set(existing + create_existing) - set(required + create_required):
        cursor.execute(f"drop role {role} cascade")
    for role in set(create_existing) - set(create_required):
        cursor.execute(f"revoke create on schema {schema} from {role}")
    for role in set(required + create_required) - set(existing + create_existing):
        cursor.execute(f"create role {role}")
        cursor.execute(f"grant usage on schema {schema} to {role}")
    for role in set(create_required) - set(create_existing):
        cursor.execute(f"grant create on schema {schema} to {role}")


def check(schema_facts, schema, usage_roles, create_roles, owner):
    schema_key = schema.lower()
    if schema_key not in schema_facts:
        return False
    if owner and owner.lower() == schema_facts[schema_key]["owner"].lower():
        return False
    if sorted(usage_roles) != sorted(schema_facts[schema_key]["usage_roles"]):
        return False
    return sorted(create_roles) == sorted(schema_facts[schema_key]["create_roles"])


def present(schema_facts, cursor, schema, usage_roles, create_roles, owner):
    schema_key = schema.lower()
    if schema_key not in schema_facts:
        query_fragments = [f"create schema {schema}"]
        if owner:
            query_fragments.append(f"authorization {owner}")
        cursor.execute(" ".join(query_fragments))
        update_roles(schema_facts, cursor, schema, [], usage_roles, [], create_roles)
        schema_facts.update(get_schema_facts(cursor, schema))
        return True
    else:
        changed = False
        if owner and owner.lower() != schema_facts[schema_key]["owner"].lower():
            raise NotSupportedError(
                f"Changing schema owner is not supported. Current owner: {schema_facts[schema_key]['owner']}."
            )
        if sorted(usage_roles) != sorted(schema_facts[schema_key]["usage_roles"]) or sorted(create_roles) != sorted(
            schema_facts[schema_key]["create_roles"]
        ):
            update_roles(
                schema_facts,
                cursor,
                schema,
                schema_facts[schema_key]["usage_roles"],
                usage_roles,
                schema_facts[schema_key]["create_roles"],
                create_roles,
            )
            changed = True
        if changed:
            schema_facts.update(get_schema_facts(cursor, schema))
        return changed


def absent(schema_facts, cursor, schema, usage_roles, create_roles):
    schema_key = schema.lower()
    if schema_key in schema_facts:
        update_roles(
            schema_facts,
            cursor,
            schema,
            schema_facts[schema_key]["usage_roles"],
            [],
            schema_facts[schema_key]["create_roles"],
            [],
        )
        try:
            cursor.execute(f"drop schema {schema_facts[schema_key]['name']} restrict")
        except pyodbc.Error as e:
            raise CannotDropError("Dropping schema failed due to dependencies.") from e
        del schema_facts[schema_key]
        return True
    else:
        return False


# module logic


def main():
    module = AnsibleModule(
        argument_spec=dict(
            schema=dict(required=True, aliases=["name"]),
            usage_roles=dict(aliases=["usage_role"]),
            create_roles=dict(aliases=["create_role"]),
            owner=dict(),
            state=dict(default="present", choices=["absent", "present"]),
            db=dict(),
            cluster=dict(default="localhost"),
            port=dict(default="5433"),
            login_user=dict(default="dbadmin"),
            login_password=dict(no_log=True),
        ),
        supports_check_mode=True,
    )

    if not pyodbc_found:
        module.fail_json(msg=missing_required_lib("pyodbc"), exception=PYODBC_IMP_ERR)

    schema = module.params["schema"]
    usage_roles = []
    if module.params["usage_roles"]:
        usage_roles = module.params["usage_roles"].split(",")
        usage_roles = [_f for _f in usage_roles if _f]
    create_roles = []
    if module.params["create_roles"]:
        create_roles = module.params["create_roles"].split(",")
        create_roles = [_f for _f in create_roles if _f]
    owner = module.params["owner"]
    state = module.params["state"]
    db = ""
    if module.params["db"]:
        db = module.params["db"]

    changed = False

    try:
        dsn = (
            "Driver=Vertica;"
            f"Server={module.params['cluster']};"
            f"Port={module.params['port']};"
            f"Database={db};"
            f"User={module.params['login_user']};"
            f"Password={module.params['login_password']};"
            f"ConnectionLoadBalance=true"
        )
        db_conn = pyodbc.connect(dsn, autocommit=True)
        cursor = db_conn.cursor()
    except Exception as e:
        module.fail_json(msg=f"Unable to connect to database: {e}.")

    try:
        schema_facts = get_schema_facts(cursor)
        if module.check_mode:
            changed = not check(schema_facts, schema, usage_roles, create_roles, owner)
        elif state == "absent":
            try:
                changed = absent(schema_facts, cursor, schema, usage_roles, create_roles)
            except pyodbc.Error as e:
                module.fail_json(msg=f"{e}", exception=traceback.format_exc())
        elif state == "present":
            try:
                changed = present(schema_facts, cursor, schema, usage_roles, create_roles, owner)
            except pyodbc.Error as e:
                module.fail_json(msg=f"{e}", exception=traceback.format_exc())
    except NotSupportedError as e:
        module.fail_json(msg=f"{e}", ansible_facts={"vertica_schemas": schema_facts})
    except CannotDropError as e:
        module.fail_json(msg=f"{e}", ansible_facts={"vertica_schemas": schema_facts})
    except SystemExit:
        # avoid catching this on python 2.4
        raise
    except Exception as e:
        module.fail_json(msg=f"{e}", exception=traceback.format_exc())

    module.exit_json(changed=changed, schema=schema, ansible_facts={"vertica_schemas": schema_facts})


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2016-2017 Hewlett Packard Enterprise Development LP
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: oneview_network_set_info
short_description: Retrieve information about the OneView Network Sets
description:
  - Retrieve information about the Network Sets from OneView.
requirements:
  - hpOneView >= 2.0.1
author:
  - Felipe Bulsoni (@fgbulsoni)
  - Thiago Miotto (@tmiotto)
  - Adriane Cardozo (@adriane-cardozo)
attributes:
  check_mode:
    version_added: 3.3.0
    # This was backported to 2.5.4 and 1.3.11 as well, since this was a bugfix
options:
  name:
    description:
      - Network Set name.
    type: str

  options:
    description:
      - 'List with options to gather information about Network Set. Option allowed: V(withoutEthernet). The option V(withoutEthernet)
        retrieves the list of network_sets excluding Ethernet networks.'
    type: list
    elements: str

extends_documentation_fragment:
  - community.general._oneview
  - community.general._oneview.factsparams
  - community.general._attributes
  - community.general._attributes.info_module
"""

EXAMPLES = r"""
- name: Gather information about all Network Sets
  community.general.oneview_network_set_info:
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
  no_log: true
  delegate_to: localhost
  register: result

- name: Print fetched information about Network Sets
  ansible.builtin.debug:
    msg: "{{ result.network_sets }}"

- name: Gather paginated, filtered and sorted information about Network Sets
  community.general.oneview_network_set_info:
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
    params:
      start: 0
      count: 3
      sort: 'name:descending'
      filter: name='netset001'
  no_log: true
  delegate_to: localhost
  register: result

- name: Print fetched information about paginated, filtered and sorted list of Network Sets
  ansible.builtin.debug:
    msg: "{{ result.network_sets }}"

- name: Gather information about all Network Sets, excluding Ethernet networks
  community.general.oneview_network_set_info:
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
    options:
      - withoutEthernet
  no_log: true
  delegate_to: localhost
  register: result

- name: Print fetched information about Network Sets, excluding Ethernet networks
  ansible.builtin.debug:
    msg: "{{ result.network_sets }}"

- name: Gather information about a Network Set by name
  community.general.oneview_network_set_info:
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
    name: Name of the Network Set
  no_log: true
  delegate_to: localhost
  register: result

- name: Print fetched information about Network Set found by name
  ansible.builtin.debug:
    msg: "{{ result.network_sets }}"

- name: Gather information about a Network Set by name, excluding Ethernet networks
  community.general.oneview_network_set_info:
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
    name: Name of the Network Set
    options:
      - withoutEthernet
  no_log: true
  delegate_to: localhost
  register: result

- name: Print fetched information about Network Set found by name, excluding Ethernet networks
  ansible.builtin.debug:
    msg: "{{ result.network_sets }}"
"""

RETURN = r"""
network_sets:
  description: Has all the OneView information about the Network Sets.
  returned: Always, but can be empty.
  type: dict
"""

from ansible_collections.community.general.plugins.module_utils._oneview import OneViewModuleBase


class NetworkSetInfoModule(OneViewModuleBase):
    argument_spec = dict(
        name=dict(type="str"),
        options=dict(type="list", elements="str"),
        params=dict(type="dict"),
    )

    def __init__(self):
        super().__init__(
            additional_arg_spec=self.argument_spec,
            supports_check_mode=True,
        )

    def execute_module(self):
        name = self.module.params.get("name")

        if "withoutEthernet" in self.options:
            filter_by_name = f"\"'name'='{name}'\"" if name else ""
            network_sets = self.oneview_client.network_sets.get_all_without_ethernet(filter=filter_by_name)
        elif name:
            network_sets = self.oneview_client.network_sets.get_by("name", name)
        else:
            network_sets = self.oneview_client.network_sets.get_all(**self.facts_params)

        return dict(changed=False, network_sets=network_sets)


def main():
    NetworkSetInfoModule().run()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2018, Johannes Brunswicker <johannes.brunswicker@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: utm_dns_host

author:
  - Johannes Brunswicker (@MatrixCrawler)

short_description: Create, update or destroy DNS entry in Sophos UTM

description:
  - Create, update or destroy a DNS entry in SOPHOS UTM.
  - This module needs to have the REST Ability of the UTM to be activated.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  name:
    type: str
    description:
      - The name of the object that identifies the entry.
    required: true
  address:
    type: str
    description:
      - The IPV4 Address of the entry. Can be left empty for automatic resolving.
    default: 0.0.0.0
  address6:
    type: str
    description:
      - The IPV6 Address of the entry. Can be left empty for automatic resolving.
    default: "::"
  comment:
    type: str
    description:
      - An optional comment to add to the DNS host object.
    default: ''
  hostname:
    type: str
    description:
      - The hostname for the DNS host object.
  interface:
    type: str
    description:
      - The reference name of the interface to use. If not provided the default interface is used.
    default: ''
  resolved:
    description:
      - Whether the hostname's ipv4 address is already resolved or not.
    default: false
    type: bool
  resolved6:
    description:
      - Whether the hostname's ipv6 address is already resolved or not.
    default: false
    type: bool
  timeout:
    type: int
    description:
      - The timeout for the UTM to resolve the IP address for the hostname again.
    default: 0

extends_documentation_fragment:
  - community.general._utm
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Create UTM dns host entry
  community.general.utm_dns_host:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestDNSEntry
    hostname: testentry.some.tld
    state: present

- name: Remove UTM dns host entry
  community.general.utm_dns_host:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestDNSEntry
    state: absent
"""

RETURN = r"""
result:
  description: The utm object that was created.
  returned: success
  type: complex
  contains:
    _ref:
      description: The reference name of the object.
      type: str
    _locked:
      description: Whether or not the object is currently locked.
      type: bool
    name:
      description: The name of the object.
      type: str
    address:
      description: The ipv4 address of the object.
      type: str
    address6:
      description: The ipv6 address of the object.
      type: str
    comment:
      description: The comment string.
      type: str
    hostname:
      description: The hostname of the object.
      type: str
    interface:
      description: The reference name of the interface the object is associated with.
      type: str
    resolved:
      description: Whether the ipv4 address is resolved or not.
      type: bool
    resolved6:
      description: Whether the ipv6 address is resolved or not.
      type: bool
    timeout:
      description: The timeout until a new resolving is attempted.
      type: int
"""

from ansible_collections.community.general.plugins.module_utils._utm_utils import UTM, UTMModule


def main():
    endpoint = "network/dns_host"
    key_to_check_for_changes = ["comment", "hostname", "interface"]
    module = UTMModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
            address=dict(type="str", default="0.0.0.0"),
            address6=dict(type="str", default="::"),
            comment=dict(type="str", default=""),
            hostname=dict(type="str"),
            interface=dict(type="str", default=""),
            resolved=dict(type="bool", default=False),
            resolved6=dict(type="bool", default=False),
            timeout=dict(type="int", default=0),
        )
    )
    try:
        UTM(module, endpoint, key_to_check_for_changes).execute()
    except Exception as e:
        module.fail_json(msg=f"{e}")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pingdom
short_description: Pause/unpause Pingdom alerts
description:
  - This module lets you pause/unpause Pingdom alerts.
author:
  - "Dylan Silva (@thaumos)"
  - "Justin Johns (!UNKNOWN)"
requirements:
  - "This pingdom python library: U(https://github.com/mbabineau/pingdom-python)"
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  state:
    type: str
    description:
      - Define whether or not the check should be running or paused.
    required: true
    choices: ["running", "paused", "started", "stopped"]
  checkid:
    type: str
    description:
      - Pingdom ID of the check.
    required: true
  uid:
    type: str
    description:
      - Pingdom user ID.
    required: true
  passwd:
    type: str
    description:
      - Pingdom user password.
    required: true
  key:
    type: str
    description:
      - Pingdom API key.
    required: true
notes:
  - This module does not yet have support to add/remove checks.
"""

EXAMPLES = r"""
- name: Pause the check with the ID of 12345
  community.general.pingdom:
    uid: example@example.com
    passwd: password123
    key: apipassword123
    checkid: 12345
    state: paused

- name: Unpause the check with the ID of 12345
  community.general.pingdom:
    uid: example@example.com
    passwd: password123
    key: apipassword123
    checkid: 12345
    state: running
"""

import traceback

PINGDOM_IMP_ERR = None
try:
    import pingdom

    HAS_PINGDOM = True
except Exception:
    PINGDOM_IMP_ERR = traceback.format_exc()
    HAS_PINGDOM = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib


def pause(checkid, uid, passwd, key):
    c = pingdom.PingdomConnection(uid, passwd, key)
    c.modify_check(checkid, paused=True)
    check = c.get_check(checkid)
    name = check.name
    result = check.status
    # if result != "paused":             # api output buggy - accept raw exception for now
    #    return (True, name, result)
    return (False, name, result)


def unpause(checkid, uid, passwd, key):
    c = pingdom.PingdomConnection(uid, passwd, key)
    c.modify_check(checkid, paused=False)
    check = c.get_check(checkid)
    name = check.name
    result = check.status
    # if result != "up":                 # api output buggy - accept raw exception for now
    #    return (True, name, result)
    return (False, name, result)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(required=True, choices=["running", "paused", "started", "stopped"]),
            checkid=dict(required=True),
            uid=dict(required=True),
            passwd=dict(required=True, no_log=True),
            key=dict(required=True, no_log=True),
        )
    )

    if not HAS_PINGDOM:
        module.fail_json(msg=missing_required_lib("pingdom"), exception=PINGDOM_IMP_ERR)

    checkid = module.params["checkid"]
    state = module.params["state"]
    uid = module.params["uid"]
    passwd = module.params["passwd"]
    key = module.params["key"]

    if state == "paused" or state == "stopped":
        (rc, name, result) = pause(checkid, uid, passwd, key)

    if state == "running" or state == "started":
        (rc, name, result) = unpause(checkid, uid, passwd, key)

    if rc != 0:
        module.fail_json(checkid=checkid, name=name, status=result)

    module.exit_json(checkid=checkid, name=name, status=result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2021, Phillipe Smith <phsmithcc@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: rundeck_job_executions_info
short_description: Query executions for a Rundeck job
description:
  - This module gets the list of executions for a specified Rundeck job.
author: "Phillipe Smith (@phsmith)"
version_added: 3.8.0
options:
  job_id:
    type: str
    description:
      - The job unique ID.
    required: true
  status:
    type: str
    description:
      - The job status to filter.
    choices: [succeeded, failed, aborted, running]
  max:
    type: int
    description:
      - Max results to return.
    default: 20
  offset:
    type: int
    description:
      - The start point to return the results.
    default: 0
extends_documentation_fragment:
  - community.general._rundeck
  - ansible.builtin.url
  - community.general._attributes
  - community.general._attributes.info_module
"""

EXAMPLES = r"""
- name: Get Rundeck job executions info
  community.general.rundeck_job_executions_info:
    url: "https://rundeck.example.org"
    api_version: 39
    api_token: "mytoken"
    job_id: "xxxxxxxxxxxxxxxxx"
  register: rundeck_job_executions_info

- name: Show Rundeck job executions info
  ansible.builtin.debug:
    var: rundeck_job_executions_info.executions
"""

RETURN = r"""
paging:
  description: Results pagination info.
  returned: success
  type: dict
  contains:
    count:
      description: Number of results in the response.
      type: int
      returned: success
    total:
      description: Total number of results.
      type: int
      returned: success
    offset:
      description: Offset from first of all results.
      type: int
      returned: success
    max:
      description: Maximum number of results per page.
      type: int
      returned: success
  sample:
    {
      "count": 20,
      "total": 100,
      "offset": 0,
      "max": 20
    }
executions:
  description: Job executions list.
  returned: always
  type: list
  elements: dict
  sample:
    [
      {
        "id": 1,
        "href": "https://rundeck.example.org/api/39/execution/1",
        "permalink": "https://rundeck.example.org/project/myproject/execution/show/1",
        "status": "succeeded",
        "project": "myproject",
        "executionType": "user",
        "user": "admin",
        "date-started": {
          "unixtime": 1633525515026,
          "date": "2021-10-06T13:05:15Z"
        },
        "date-ended": {
          "unixtime": 1633525518386,
          "date": "2021-10-06T13:05:18Z"
        },
        "job": {
          "id": "697af0c4-72d3-4c15-86a3-b5bfe3c6cb6a",
          "averageDuration": 6381,
          "name": "Test",
          "group": "",
          "project": "myproject",
          "description": "",
          "options": {
            "exit_code": "0"
          },
          "href": "https://rundeck.example.org/api/39/job/697af0c4-72d3-4c15-86a3-b5bfe3c6cb6a",
          "permalink": "https://rundeck.example.org/project/myproject/job/show/697af0c4-72d3-4c15-86a3-b5bfe3c6cb6a"
        },
        "description": "Plugin[com.batix.rundeck.plugins.AnsiblePlaybookInlineWorkflowStep, nodeStep: false]",
        "argstring": "-exit_code 0",
        "serverUUID": "5b9a1438-fa3a-457e-b254-8f3d70338068"
      }
    ]
"""

from urllib.parse import quote

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._rundeck import api_argument_spec, api_request


class RundeckJobExecutionsInfo:
    def __init__(self, module):
        self.module = module
        self.url = self.module.params["url"]
        self.api_version = self.module.params["api_version"]
        self.job_id = self.module.params["job_id"]
        self.offset = self.module.params["offset"]
        self.max = self.module.params["max"]
        self.status = self.module.params["status"] or ""

    def job_executions(self):
        response, info = api_request(
            module=self.module,
            endpoint=f"job/{quote(self.job_id)}/executions?offset={self.offset}&max={self.max}&status={self.status}",
            method="GET",
        )

        if info["status"] != 200:
            self.module.fail_json(msg=info["msg"], executions=response)

        self.module.exit_json(msg="Executions info result", **response)


def main():
    argument_spec = api_argument_spec()
    argument_spec.update(
        dict(
            job_id=dict(required=True, type="str"),
            offset=dict(type="int", default=0),
            max=dict(type="int", default=20),
            status=dict(type="str", choices=["succeeded", "failed", "aborted", "running"]),
        )
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    if module.params["api_version"] < 14:
        module.fail_json(msg="API version should be at least 14")

    rundeck = RundeckJobExecutionsInfo(module)
    rundeck.job_executions()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, Tim Rightnour <thegarbledone@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: syslogger
short_description: Log messages in the syslog
description:
  - Uses syslog to add log entries to the host.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  msg:
    type: str
    description:
      - This is the message to place in syslog.
    required: true
  priority:
    type: str
    description:
      - Set the log priority.
    choices: ["emerg", "alert", "crit", "err", "warning", "notice", "info", "debug"]
    default: "info"
  facility:
    type: str
    description:
      - Set the log facility.
    choices:
      - kern
      - user
      - mail
      - daemon
      - auth
      - lpr
      - news
      - uucp
      - cron
      - syslog
      - local0
      - local1
      - local2
      - local3
      - local4
      - local5
      - local6
      - local7
    default: "daemon"
  log_pid:
    description:
      - Log the PID in brackets.
    type: bool
    default: false
  ident:
    description:
      - Specify the name of application name which is sending the log to syslog.
    type: str
    default: 'ansible_syslogger'
    version_added: '0.2.0'
author:
  - Tim Rightnour (@garbled1)
"""

EXAMPLES = r"""
- name: Simple Usage
  community.general.syslogger:
    msg: "I will end up as daemon.info"

- name: Send a log message with err priority and user facility with log_pid
  community.general.syslogger:
    msg: "Hello from Ansible"
    priority: "err"
    facility: "user"
    log_pid: true

- name: Specify the name of application which is sending log message
  community.general.syslogger:
    ident: "MyApp"
    msg: "I want to believe"
    priority: "alert"
"""

RETURN = r"""
ident:
  description: Name of application sending the message to log.
  returned: always
  type: str
  sample: "ansible_syslogger"
  version_added: '0.2.0'
priority:
  description: Priority level.
  returned: always
  type: str
  sample: "daemon"
facility:
  description: Syslog facility.
  returned: always
  type: str
  sample: "info"
log_pid:
  description: Log PID status.
  returned: always
  type: bool
  sample: true
msg:
  description: Message sent to syslog.
  returned: always
  type: str
  sample: "Hello from Ansible"
"""

import syslog
import traceback

from ansible.module_utils.basic import AnsibleModule


def get_facility(facility):
    return {
        "kern": syslog.LOG_KERN,
        "user": syslog.LOG_USER,
        "mail": syslog.LOG_MAIL,
        "daemon": syslog.LOG_DAEMON,
        "auth": syslog.LOG_AUTH,
        "lpr": syslog.LOG_LPR,
        "news": syslog.LOG_NEWS,
        "uucp": syslog.LOG_UUCP,
        "cron": syslog.LOG_CRON,
        "syslog": syslog.LOG_SYSLOG,
        "local0": syslog.LOG_LOCAL0,
        "local1": syslog.LOG_LOCAL1,
        "local2": syslog.LOG_LOCAL2,
        "local3": syslog.LOG_LOCAL3,
        "local4": syslog.LOG_LOCAL4,
        "local5": syslog.LOG_LOCAL5,
        "local6": syslog.LOG_LOCAL6,
        "local7": syslog.LOG_LOCAL7,
    }.get(facility, syslog.LOG_DAEMON)


def get_priority(priority):
    return {
        "emerg": syslog.LOG_EMERG,
        "alert": syslog.LOG_ALERT,
        "crit": syslog.LOG_CRIT,
        "err": syslog.LOG_ERR,
        "warning": syslog.LOG_WARNING,
        "notice": syslog.LOG_NOTICE,
        "info": syslog.LOG_INFO,
        "debug": syslog.LOG_DEBUG,
    }.get(priority, syslog.LOG_INFO)


def main():
    # define the available arguments/parameters that a user can pass to
    # the module
    module_args = dict(
        ident=dict(type="str", default="ansible_syslogger"),
        msg=dict(type="str", required=True),
        priority=dict(
            type="str", choices=["emerg", "alert", "crit", "err", "warning", "notice", "info", "debug"], default="info"
        ),
        facility=dict(
            type="str",
            choices=[
                "kern",
                "user",
                "mail",
                "daemon",
                "auth",
                "lpr",
                "news",
                "uucp",
                "cron",
                "syslog",
                "local0",
                "local1",
                "local2",
                "local3",
                "local4",
                "local5",
                "local6",
                "local7",
            ],
            default="daemon",
        ),
        log_pid=dict(type="bool", default=False),
    )

    module = AnsibleModule(
        argument_spec=module_args,
    )

    result = dict(
        changed=False,
        ident=module.params["ident"],
        priority=module.params["priority"],
        facility=module.params["facility"],
        log_pid=module.params["log_pid"],
        msg=module.params["msg"],
    )

    # do the logging
    try:
        syslog.openlog(
            module.params["ident"],
            syslog.LOG_PID if module.params["log_pid"] else 0,
            get_facility(module.params["facility"]),
        )
        syslog.syslog(get_priority(module.params["priority"]), module.params["msg"])
        syslog.closelog()
        result["changed"] = True

    except Exception as exc:
        module.fail_json(error=f"Failed to write to syslog {exc}", exception=traceback.format_exc(), **result)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Scaleway IP management module
#
# Copyright (c) Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: scaleway_ip
short_description: Scaleway IP management module
author: Remy Leone (@remyleone)
description:
  - This module manages IP on Scaleway account U(https://developer.scaleway.com).
extends_documentation_fragment:
  - community.general._scaleway
  - community.general._attributes
  - community.general._scaleway.actiongroup_scaleway

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
  action_group:
    version_added: 11.3.0

options:
  state:
    type: str
    description:
      - Indicate desired state of the IP.
    default: present
    choices:
      - present
      - absent

  organization:
    type: str
    description:
      - Scaleway organization identifier.
      - Exactly one of O(project) and O(organization) must be specified.

  project:
    type: str
    description:
      - Project identifier.
      - Exactly one of O(project) and O(organization) must be specified.
    version_added: 12.3.0

  region:
    type: str
    description:
      - Scaleway region to use (for example par1).
    required: true
    choices:
      - ams1
      - EMEA-NL-EVS
      - ams2
      - ams3
      - par1
      - EMEA-FR-PAR1
      - par2
      - EMEA-FR-PAR2
      - par3
      - waw1
      - EMEA-PL-WAW1
      - waw2
      - waw3

  id:
    type: str
    description:
      - ID of the Scaleway IP (UUID).
  server:
    type: str
    description:
      - ID of the server you want to attach an IP to.
      - To unattach an IP do not specify this option.
  reverse:
    type: str
    description:
      - Reverse to assign to the IP.
"""

EXAMPLES = r"""
- name: Create an IP with a project ID
  community.general.scaleway_ip:
    project: '{{ project_id }}'
    state: present
    region: par1
  register: ip_creation_task

- name: Make sure IP deleted
  community.general.scaleway_ip:
    id: '{{ ip_creation_task.scaleway_ip.id }}'
    state: absent
    region: par1

- name: Create an IP in the default project with an organization ID (deprecated)
  community.general.scaleway_ip:
    organization: '{{ scw_org }}'
    state: present
    region: par1
  register: ip_creation_task
"""

RETURN = r"""
data:
  description: This is only present when O(state=present).
  returned: when O(state=present)
  type: dict
  sample:
    {
      "ips": [
        {
          "organization": "951df375-e094-4d26-97c1-ba548eeb9c42",
          "reverse": null,
          "id": "dd9e8df6-6775-4863-b517-e0b0ee3d7477",
          "server": {
            "id": "3f1568ca-b1a2-4e98-b6f7-31a0588157f1",
            "name": "ansible_tuto-1"
          },
          "address": "212.47.232.136"
        }
      ]
    }
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._scaleway import (
    SCALEWAY_LOCATION,
    Scaleway,
    scaleway_argument_spec,
)


def ip_attributes_should_be_changed(api, target_ip, wished_ip):
    patch_payload = {}

    if target_ip["reverse"] != wished_ip["reverse"]:
        patch_payload["reverse"] = wished_ip["reverse"]

    # IP is assigned to a server
    if target_ip["server"] is None and wished_ip["server"]:
        patch_payload["server"] = wished_ip["server"]

    # IP is unassigned to a server
    try:
        if target_ip["server"]["id"] and wished_ip["server"] is None:
            patch_payload["server"] = wished_ip["server"]
    except (TypeError, KeyError):
        pass

    # IP is migrated between 2 different servers
    try:
        if target_ip["server"]["id"] != wished_ip["server"]:
            patch_payload["server"] = wished_ip["server"]
    except (TypeError, KeyError):
        pass

    return patch_payload


def payload_from_wished_ip(wished_ip):
    return {k: v for k, v in wished_ip.items() if k != "id" and v is not None}


def present_strategy(api, wished_ip):
    changed = False

    response = api.get("ips")
    if not response.ok:
        api.module.fail_json(msg=f"Error getting IPs [{response.status_code}: {response.json['message']}]")

    ips_list = response.json["ips"]
    ip_lookup = {ip["id"]: ip for ip in ips_list}

    if wished_ip["id"] not in ip_lookup.keys():
        changed = True
        if api.module.check_mode:
            return changed, {"status": "An IP would be created."}

        # Create IP
        creation_response = api.post("/ips", data=payload_from_wished_ip(wished_ip))

        if not creation_response.ok:
            msg = f"Error during ip creation: {creation_response.info['msg']}: '{creation_response.json['message']}' ({creation_response.json})"
            api.module.fail_json(msg=msg)
        return changed, creation_response.json["ip"]

    target_ip = ip_lookup[wished_ip["id"]]
    patch_payload = ip_attributes_should_be_changed(api=api, target_ip=target_ip, wished_ip=wished_ip)

    if not patch_payload:
        return changed, target_ip

    changed = True
    if api.module.check_mode:
        return changed, {"status": "IP attributes would be changed."}

    ip_patch_response = api.patch(path=f"ips/{target_ip['id']}", data=patch_payload)

    if not ip_patch_response.ok:
        api.module.fail_json(
            msg=f"Error during IP attributes update: [{ip_patch_response.status_code}: {ip_patch_response.json['message']}]"
        )

    return changed, ip_patch_response.json["ip"]


def absent_strategy(api, wished_ip):
    response = api.get("ips")
    changed = False

    status_code = response.status_code
    ips_json = response.json
    ips_list = ips_json["ips"]

    if not response.ok:
        api.module.fail_json(msg=f"Error getting IPs [{status_code}: {response.json['message']}]")

    ip_lookup = {ip["id"]: ip for ip in ips_list}
    if wished_ip["id"] not in ip_lookup.keys():
        return changed, {}

    changed = True
    if api.module.check_mode:
        return changed, {"status": "IP would be destroyed"}

    response = api.delete(f"/ips/{wished_ip['id']}")
    if not response.ok:
        api.module.fail_json(msg=f"Error deleting IP [{response.status_code}: {response.json}]")

    return changed, response.json


def core(module):
    wished_ip = {
        "organization": module.params["organization"],
        "project": module.params["project"],
        "reverse": module.params["reverse"],
        "id": module.params["id"],
        "server": module.params["server"],
    }

    region = module.params["region"]
    module.params["api_url"] = SCALEWAY_LOCATION[region]["api_endpoint"]

    api = Scaleway(module=module)
    if module.params["state"] == "absent":
        changed, summary = absent_strategy(api=api, wished_ip=wished_ip)
    else:
        changed, summary = present_strategy(api=api, wished_ip=wished_ip)
    module.exit_json(changed=changed, scaleway_ip=summary)


def main():
    argument_spec = scaleway_argument_spec()
    argument_spec.update(
        dict(
            state=dict(default="present", choices=["absent", "present"]),
            organization=dict(),
            project=dict(),
            server=dict(),
            reverse=dict(),
            region=dict(required=True, choices=list(SCALEWAY_LOCATION.keys())),
            id=dict(),
        )
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ("organization", "project"),
        ],
        required_one_of=[
            ("organization", "project"),
        ],
    )

    core(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2016, Andrew Gaffney <andrew@agaffney.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: openwrt_init
author:
  - "Andrew Gaffney (@agaffney)"
short_description: Manage services on OpenWrt
description:
  - Controls OpenWrt services on remote hosts.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    type: str
    description:
      - Name of the service.
    required: true
    aliases: ['service']
  state:
    type: str
    description:
      - V(started)/V(stopped) are idempotent actions that do not run commands unless necessary.
      - V(restarted) always bounces the service.
      - V(reloaded) always reloads.
    choices: ['started', 'stopped', 'restarted', 'reloaded']
  enabled:
    description:
      - Whether the service should start on boot. B(At least one) of O(state) and O(enabled) are required.
    type: bool
  pattern:
    type: str
    description:
      - If the service does not respond to the C(running) command, name a substring to look for as would be found in the output
        of the C(ps) command as a stand-in for a C(running) result. If the string is found, the service is assumed to be running.
notes:
  - One option other than O(name) is required.
requirements:
  - An OpenWrt system (with python)
"""

EXAMPLES = r"""
- name: Start service httpd, if not running
  community.general.openwrt_init:
    state: started
    name: httpd

- name: Stop service cron, if running
  community.general.openwrt_init:
    name: cron
    state: stopped

- name: Reload service httpd, in all cases
  community.general.openwrt_init:
    name: httpd
    state: reloaded

- name: Enable service httpd
  community.general.openwrt_init:
    name: httpd
    enabled: true
"""

RETURN = r"""
"""

import os

from ansible.module_utils.basic import AnsibleModule

module = None
init_script = None


# ===============================
# Check if service is enabled
def is_enabled():
    rc, dummy, dummy = module.run_command([init_script, "enabled"])
    return rc == 0


# ===========================================
# Main control flow
def main():
    global module, init_script
    # init
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True, type="str", aliases=["service"]),
            state=dict(type="str", choices=["started", "stopped", "restarted", "reloaded"]),
            enabled=dict(type="bool"),
            pattern=dict(type="str"),
        ),
        supports_check_mode=True,
        required_one_of=[("state", "enabled")],
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    # initialize
    service = module.params["name"]
    init_script = f"/etc/init.d/{service}"
    result = {
        "name": service,
        "changed": False,
    }
    # check if service exists
    if not os.path.exists(init_script):
        module.fail_json(msg=f"service {service} does not exist")

    # Enable/disable service startup at boot if requested
    if module.params["enabled"] is not None:
        # do we need to enable the service?
        enabled = is_enabled()

        # default to current state
        result["enabled"] = enabled

        # Change enable/disable if needed
        if enabled != module.params["enabled"]:
            result["changed"] = True
            action = "enable" if module.params["enabled"] else "disable"

            if not module.check_mode:
                rc, dummy, err = module.run_command([init_script, action])
                # openwrt init scripts can return a non-zero exit code on a successful 'enable'
                # command if the init script doesn't contain a STOP value, so we ignore the exit
                # code and explicitly check if the service is now in the desired state
                if is_enabled() != module.params["enabled"]:
                    module.fail_json(msg=f"Unable to {action} service {service}: {err}")

            result["enabled"] = not enabled

    if module.params["state"] is not None:
        running = False

        # check if service is currently running
        if module.params["pattern"]:
            # Find ps binary
            psbin = module.get_bin_path("ps", True)

            # this should be busybox ps, so we only want/need to the 'w' option
            rc, psout, dummy = module.run_command([psbin, "w"])
            # If rc is 0, set running as appropriate
            if rc == 0:
                lines = psout.split("\n")
                running = any((module.params["pattern"] in line and "pattern=" not in line) for line in lines)
        else:
            rc, dummy, dummy = module.run_command([init_script, "running"])
            if rc == 0:
                running = True

        # default to desired state
        result["state"] = module.params["state"]

        # determine action, if any
        action = None
        if module.params["state"] == "started":
            if not running:
                action = "start"
                result["changed"] = True
        elif module.params["state"] == "stopped":
            if running:
                action = "stop"
                result["changed"] = True
        else:
            action = module.params["state"][:-2]  # remove 'ed' from restarted/reloaded
            result["state"] = "started"
            result["changed"] = True

        if action:
            if not module.check_mode:
                rc, dummy, err = module.run_command([init_script, action])
                if rc != 0:
                    module.fail_json(msg=f"Unable to {action} service {service}: {err}")

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright 2015 WP Engine, Inc. All rights reserved.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: znode
short_description: Create, delete, retrieve, and update znodes using ZooKeeper
description:
  - Create, delete, retrieve, and update znodes using ZooKeeper.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
extends_documentation_fragment:
  - community.general._attributes
options:
  hosts:
    description:
      - A list of ZooKeeper servers (format V([server]:[port])).
    required: true
    type: str
  name:
    description:
      - The path of the znode.
    required: true
    type: str
  value:
    description:
      - The value assigned to the znode.
    type: str
  op:
    description:
      - An operation to perform. Mutually exclusive with state.
    choices: [get, wait, list]
    type: str
  state:
    description:
      - The state to enforce. Mutually exclusive with op.
    choices: [present, absent]
    type: str
  timeout:
    description:
      - The amount of time to wait for a node to appear.
    default: 300
    type: int
  recursive:
    description:
      - Recursively delete node and all its children.
    type: bool
    default: false
  auth_scheme:
    description:
      - Authentication scheme.
    choices: [digest, sasl]
    type: str
    default: "digest"
    version_added: 5.8.0
  auth_credential:
    description:
      - The authentication credential value. Depends on O(auth_scheme).
      - The format for O(auth_scheme=digest) is C(user:password), and the format for O(auth_scheme=sasl) is C(user:password).
    type: str
    version_added: 5.8.0
  use_tls:
    description:
      - Using TLS/SSL or not.
    type: bool
    default: false
    version_added: '6.5.0'
requirements:
  - kazoo >= 2.1
author: "Trey Perry (@treyperry)"
"""

EXAMPLES = r"""
- name: Creating or updating a znode with a given value
  community.general.znode:
    hosts: 'localhost:2181'
    name: /mypath
    value: myvalue
    state: present

- name: Getting the value and stat structure for a znode
  community.general.znode:
    hosts: 'localhost:2181'
    name: /mypath
    op: get

- name: Getting the value and stat structure for a znode using digest authentication
  community.general.znode:
    hosts: 'localhost:2181'
    auth_credential: 'user1:s3cr3t'
    name: /secretmypath
    op: get

- name: Listing a particular znode's children
  community.general.znode:
    hosts: 'localhost:2181'
    name: /zookeeper
    op: list

- name: Waiting 20 seconds for a znode to appear at path /mypath
  community.general.znode:
    hosts: 'localhost:2181'
    name: /mypath
    op: wait
    timeout: 20

- name: Deleting a znode at path /mypath
  community.general.znode:
    hosts: 'localhost:2181'
    name: /mypath
    state: absent

- name: Creating or updating a znode with a given value on a remote Zookeeper
  community.general.znode:
    hosts: 'my-zookeeper-node:2181'
    name: /mypath
    value: myvalue
    state: present
  delegate_to: 127.0.0.1
"""

import time
import traceback

KAZOO_IMP_ERR = None
try:
    from kazoo.client import KazooClient
    from kazoo.handlers.threading import KazooTimeoutError

    KAZOO_INSTALLED = True
except ImportError:
    KAZOO_IMP_ERR = traceback.format_exc()
    KAZOO_INSTALLED = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_bytes


def main():
    module = AnsibleModule(
        argument_spec=dict(
            hosts=dict(required=True, type="str"),
            name=dict(required=True, type="str"),
            value=dict(type="str"),
            op=dict(choices=["get", "wait", "list"]),
            state=dict(choices=["present", "absent"]),
            timeout=dict(default=300, type="int"),
            recursive=dict(default=False, type="bool"),
            auth_scheme=dict(default="digest", choices=["digest", "sasl"]),
            auth_credential=dict(type="str", no_log=True),
            use_tls=dict(default=False, type="bool"),
        ),
        supports_check_mode=False,
    )

    if not KAZOO_INSTALLED:
        module.fail_json(msg=missing_required_lib("kazoo >= 2.1"), exception=KAZOO_IMP_ERR)

    check = check_params(module.params)
    if not check["success"]:
        module.fail_json(msg=check["msg"])

    zoo = KazooCommandProxy(module)
    try:
        zoo.start()
    except KazooTimeoutError:
        module.fail_json(msg="The connection to the ZooKeeper ensemble timed out.")

    command_dict = {
        "op": {"get": zoo.get, "list": zoo.list, "wait": zoo.wait},
        "state": {"present": zoo.present, "absent": zoo.absent},
    }

    command_type = "op" if "op" in module.params and module.params["op"] is not None else "state"
    method = module.params[command_type]
    result, result_dict = command_dict[command_type][method]()
    zoo.shutdown()

    if result:
        module.exit_json(**result_dict)
    else:
        module.fail_json(**result_dict)


def check_params(params):
    if not params["state"] and not params["op"]:
        return {"success": False, "msg": "Please define an operation (op) or a state."}

    if params["state"] and params["op"]:
        return {"success": False, "msg": "Please choose an operation (op) or a state, but not both."}

    return {"success": True}


class KazooCommandProxy:
    def __init__(self, module):
        self.module = module
        self.zk = KazooClient(module.params["hosts"], use_ssl=module.params["use_tls"])

    def absent(self):
        return self._absent(self.module.params["name"])

    def exists(self, znode):
        return self.zk.exists(znode)

    def list(self):
        children = self.zk.get_children(self.module.params["name"])
        return True, {
            "count": len(children),
            "items": children,
            "msg": "Retrieved znodes in path.",
            "znode": self.module.params["name"],
        }

    def present(self):
        return self._present(self.module.params["name"], self.module.params["value"])

    def get(self):
        return self._get(self.module.params["name"])

    def shutdown(self):
        self.zk.stop()
        self.zk.close()

    def start(self):
        self.zk.start()
        if self.module.params["auth_credential"]:
            self.zk.add_auth(self.module.params["auth_scheme"], self.module.params["auth_credential"])

    def wait(self):
        return self._wait(self.module.params["name"], self.module.params["timeout"])

    def _absent(self, znode):
        if self.exists(znode):
            self.zk.delete(znode, recursive=self.module.params["recursive"])
            return True, {"changed": True, "msg": "The znode was deleted."}
        else:
            return True, {"changed": False, "msg": "The znode does not exist."}

    def _get(self, path):
        if self.exists(path):
            value, zstat = self.zk.get(path)
            stat_dict = {}
            for i in dir(zstat):
                if not i.startswith("_"):
                    attr = getattr(zstat, i)
                    if isinstance(attr, (int, str)):
                        stat_dict[i] = attr
            result = True, {"msg": "The node was retrieved.", "znode": path, "value": value, "stat": stat_dict}
        else:
            result = False, {"msg": "The requested node does not exist."}

        return result

    def _present(self, path, value):
        if self.exists(path):
            (current_value, zstat) = self.zk.get(path)
            if value != current_value:
                self.zk.set(path, to_bytes(value))
                return True, {"changed": True, "msg": "Updated the znode value.", "znode": path, "value": value}
            else:
                return True, {"changed": False, "msg": "No changes were necessary.", "znode": path, "value": value}
        else:
            self.zk.create(path, to_bytes(value), makepath=True)
            return True, {"changed": True, "msg": "Created a new znode.", "znode": path, "value": value}

    def _wait(self, path, timeout, interval=5):
        lim = time.time() + timeout

        while time.time() < lim:
            if self.exists(path):
                return True, {
                    "msg": "The node appeared before the configured timeout.",
                    "znode": path,
                    "timeout": timeout,
                }
            else:
                time.sleep(interval)

        return False, {
            "msg": "The node did not appear before the operation timed out.",
            "timeout": timeout,
            "znode": path,
        }


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2016, Adfinis SyGroup AG
# @keachi
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: udm_share
author:
  - keachi (@keachi)
short_description: Manage samba shares on a univention corporate server
description:
  - This module allows to manage samba shares on a univention corporate server (UCS). It uses the Python API of the UCS to
    create a new object or edit it.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: partial
options:
  state:
    default: "present"
    choices: [present, absent]
    description:
      - Whether the share is present or not.
    type: str
  name:
    required: true
    description:
      - Name.
    type: str
  host:
    description:
      - Host FQDN (server which provides the share), for example V({{ ansible_fqdn }}). Required if O(state=present).
    type: str
  path:
    description:
      - Directory on the providing server, for example V(/home). Required if O(state=present).
    type: path
  sambaName:
    description:
      - Windows name. Required if O(state=present).
    type: str
    aliases: [samba_name]
  ou:
    required: true
    description:
      - Organisational unit, inside the LDAP Base DN.
    type: str
  owner:
    default: '0'
    description:
      - Directory owner of the share's root directory.
    type: str
  group:
    default: '0'
    description:
      - Directory owner group of the share's root directory.
    type: str
  directorymode:
    default: '00755'
    description:
      - Permissions for the share's root directory.
    type: str
  root_squash:
    default: true
    description:
      - Modify user ID for root user (root squashing).
    type: bool
  subtree_checking:
    default: true
    description:
      - Subtree checking.
    type: bool
  sync:
    default: 'sync'
    description:
      - NFS synchronisation.
    type: str
  writeable:
    default: true
    description:
      - NFS write access.
    type: bool
  sambaBlockSize:
    description:
      - Blocking size.
    type: str
    aliases: [samba_block_size]
  sambaBlockingLocks:
    default: true
    description:
      - Blocking locks.
    type: bool
    aliases: [samba_blocking_locks]
  sambaBrowseable:
    description:
      - Show in Windows network environment.
    type: bool
    default: true
    aliases: [samba_browsable]
  sambaCreateMode:
    default: '0744'
    description:
      - File mode.
    type: str
    aliases: [samba_create_mode]
  sambaCscPolicy:
    default: 'manual'
    description:
      - Client-side caching policy.
    type: str
    aliases: [samba_csc_policy]
  sambaCustomSettings:
    default: []
    description:
      - Option name in C(smb.conf) and its value.
    type: list
    elements: dict
    aliases: [samba_custom_settings]
  sambaDirectoryMode:
    default: '0755'
    description:
      - Directory mode.
    type: str
    aliases: [samba_directory_mode]
  sambaDirectorySecurityMode:
    default: '0777'
    description:
      - Directory security mode.
    type: str
    aliases: [samba_directory_security_mode]
  sambaDosFilemode:
    default: false
    description:
      - Users with write access may modify permissions.
    type: bool
    aliases: [samba_dos_filemode]
  sambaFakeOplocks:
    default: false
    description:
      - Fake oplocks.
    type: bool
    aliases: [samba_fake_oplocks]
  sambaForceCreateMode:
    default: false
    description:
      - Force file mode.
    type: bool
    aliases: [samba_force_create_mode]
  sambaForceDirectoryMode:
    default: false
    description:
      - Force directory mode.
    type: bool
    aliases: [samba_force_directory_mode]
  sambaForceDirectorySecurityMode:
    default: false
    description:
      - Force directory security mode.
    type: bool
    aliases: [samba_force_directory_security_mode]
  sambaForceGroup:
    description:
      - Force group.
    type: str
    aliases: [samba_force_group]
  sambaForceSecurityMode:
    default: false
    description:
      - Force security mode.
    type: bool
    aliases: [samba_force_security_mode]
  sambaForceUser:
    description:
      - Force user.
    type: str
    aliases: [samba_force_user]
  sambaHideFiles:
    description:
      - Hide files.
    type: str
    aliases: [samba_hide_files]
  sambaHideUnreadable:
    default: false
    description:
      - Hide unreadable files/directories.
    type: bool
    aliases: [samba_hide_unreadable]
  sambaHostsAllow:
    default: []
    description:
      - Allowed host/network.
    type: list
    elements: str
    aliases: [samba_hosts_allow]
  sambaHostsDeny:
    default: []
    description:
      - Denied host/network.
    type: list
    elements: str
    aliases: [samba_hosts_deny]
  sambaInheritAcls:
    default: true
    description:
      - Inherit ACLs.
    type: bool
    aliases: [samba_inherit_acls]
  sambaInheritOwner:
    default: false
    description:
      - Create files/directories with the owner of the parent directory.
    type: bool
    aliases: [samba_inherit_owner]
  sambaInheritPermissions:
    default: false
    description:
      - Create files/directories with permissions of the parent directory.
    type: bool
    aliases: [samba_inherit_permissions]
  sambaInvalidUsers:
    description:
      - Invalid users or groups.
    type: str
    aliases: [samba_invalid_users]
  sambaLevel2Oplocks:
    default: true
    description:
      - Level 2 oplocks.
    type: bool
    aliases: [samba_level_2_oplocks]
  sambaLocking:
    default: true
    description:
      - Locking.
    type: bool
    aliases: [samba_locking]
  sambaMSDFSRoot:
    default: false
    description:
      - MSDFS root.
    type: bool
    aliases: [samba_msdfs_root]
  sambaNtAclSupport:
    default: true
    description:
      - NT ACL support.
    type: bool
    aliases: [samba_nt_acl_support]
  sambaOplocks:
    default: true
    description:
      - Oplocks.
    type: bool
    aliases: [samba_oplocks]
  sambaPostexec:
    description:
      - Postexec script.
    type: str
    aliases: [samba_postexec]
  sambaPreexec:
    description:
      - Preexec script.
    type: str
    aliases: [samba_preexec]
  sambaPublic:
    default: false
    description:
      - Allow anonymous read-only access with a guest user.
    type: bool
    aliases: [samba_public]
  sambaSecurityMode:
    default: '0777'
    description:
      - Security mode.
    type: str
    aliases: [samba_security_mode]
  sambaStrictLocking:
    default: 'Auto'
    description:
      - Strict locking.
    type: str
    aliases: [samba_strict_locking]
  sambaVFSObjects:
    description:
      - VFS objects.
    type: str
    aliases: [samba_vfs_objects]
  sambaValidUsers:
    description:
      - Valid users or groups.
    type: str
    aliases: [samba_valid_users]
  sambaWriteList:
    description:
      - Restrict write access to these users/groups.
    type: str
    aliases: [samba_write_list]
  sambaWriteable:
    default: true
    description:
      - Samba write access.
    type: bool
    aliases: [samba_writeable]
  nfs_hosts:
    default: []
    description:
      - Only allow access for this host, IP address or network.
    type: list
    elements: str
  nfsCustomSettings:
    default: []
    description:
      - Option name in exports file.
    type: list
    elements: str
    aliases: [nfs_custom_settings]
"""


EXAMPLES = r"""
- name: Create a share named home on the server ucs.example.com with the path /home
  community.general.udm_share:
    name: home
    path: /home
    host: ucs.example.com
    sambaName: Home
"""


RETURN = """#"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._univention_umc import (
    base_dn,
    ldap_search,
    umc_module_for_add,
    umc_module_for_edit,
)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True, type="str"),
            ou=dict(required=True, type="str"),
            owner=dict(type="str", default="0"),
            group=dict(type="str", default="0"),
            path=dict(type="path"),
            directorymode=dict(type="str", default="00755"),
            host=dict(type="str"),
            root_squash=dict(type="bool", default=True),
            subtree_checking=dict(type="bool", default=True),
            sync=dict(type="str", default="sync"),
            writeable=dict(type="bool", default=True),
            sambaBlockSize=dict(type="str", aliases=["samba_block_size"]),
            sambaBlockingLocks=dict(type="bool", aliases=["samba_blocking_locks"], default=True),
            sambaBrowseable=dict(type="bool", aliases=["samba_browsable"], default=True),
            sambaCreateMode=dict(type="str", aliases=["samba_create_mode"], default="0744"),
            sambaCscPolicy=dict(type="str", aliases=["samba_csc_policy"], default="manual"),
            sambaCustomSettings=dict(type="list", elements="dict", aliases=["samba_custom_settings"], default=[]),
            sambaDirectoryMode=dict(type="str", aliases=["samba_directory_mode"], default="0755"),
            sambaDirectorySecurityMode=dict(type="str", aliases=["samba_directory_security_mode"], default="0777"),
            sambaDosFilemode=dict(type="bool", aliases=["samba_dos_filemode"], default=False),
            sambaFakeOplocks=dict(type="bool", aliases=["samba_fake_oplocks"], default=False),
            sambaForceCreateMode=dict(type="bool", aliases=["samba_force_create_mode"], default=False),
            sambaForceDirectoryMode=dict(type="bool", aliases=["samba_force_directory_mode"], default=False),
            sambaForceDirectorySecurityMode=dict(
                type="bool", aliases=["samba_force_directory_security_mode"], default=False
            ),
            sambaForceGroup=dict(type="str", aliases=["samba_force_group"]),
            sambaForceSecurityMode=dict(type="bool", aliases=["samba_force_security_mode"], default=False),
            sambaForceUser=dict(type="str", aliases=["samba_force_user"]),
            sambaHideFiles=dict(type="str", aliases=["samba_hide_files"]),
            sambaHideUnreadable=dict(type="bool", aliases=["samba_hide_unreadable"], default=False),
            sambaHostsAllow=dict(type="list", elements="str", aliases=["samba_hosts_allow"], default=[]),
            sambaHostsDeny=dict(type="list", elements="str", aliases=["samba_hosts_deny"], default=[]),
            sambaInheritAcls=dict(type="bool", aliases=["samba_inherit_acls"], default=True),
            sambaInheritOwner=dict(type="bool", aliases=["samba_inherit_owner"], default=False),
            sambaInheritPermissions=dict(type="bool", aliases=["samba_inherit_permissions"], default=False),
            sambaInvalidUsers=dict(type="str", aliases=["samba_invalid_users"]),
            sambaLevel2Oplocks=dict(type="bool", aliases=["samba_level_2_oplocks"], default=True),
            sambaLocking=dict(type="bool", aliases=["samba_locking"], default=True),
            sambaMSDFSRoot=dict(type="bool", aliases=["samba_msdfs_root"], default=False),
            sambaName=dict(type="str", aliases=["samba_name"]),
            sambaNtAclSupport=dict(type="bool", aliases=["samba_nt_acl_support"], default=True),
            sambaOplocks=dict(type="bool", aliases=["samba_oplocks"], default=True),
            sambaPostexec=dict(type="str", aliases=["samba_postexec"]),
            sambaPreexec=dict(type="str", aliases=["samba_preexec"]),
            sambaPublic=dict(type="bool", aliases=["samba_public"], default=False),
            sambaSecurityMode=dict(type="str", aliases=["samba_security_mode"], default="0777"),
            sambaStrictLocking=dict(type="str", aliases=["samba_strict_locking"], default="Auto"),
            sambaVFSObjects=dict(type="str", aliases=["samba_vfs_objects"]),
            sambaValidUsers=dict(type="str", aliases=["samba_valid_users"]),
            sambaWriteList=dict(type="str", aliases=["samba_write_list"]),
            sambaWriteable=dict(type="bool", aliases=["samba_writeable"], default=True),
            nfs_hosts=dict(type="list", elements="str", default=[]),
            nfsCustomSettings=dict(type="list", elements="str", aliases=["nfs_custom_settings"], default=[]),
            state=dict(default="present", choices=["present", "absent"], type="str"),
        ),
        supports_check_mode=True,
        required_if=([("state", "present", ["path", "host", "sambaName"])]),
    )
    name = module.params["name"]
    state = module.params["state"]
    changed = False
    diff = None

    obj = list(ldap_search(f"(&(objectClass=univentionShare)(cn={name}))", attr=["cn"]))

    exists = bool(len(obj))
    container = f"cn=shares,ou={module.params['ou']},{base_dn()}"
    dn = f"cn={name},{container}"

    if state == "present":
        try:
            if not exists:
                obj = umc_module_for_add("shares/share", container)
            else:
                obj = umc_module_for_edit("shares/share", dn)

            module.params["printablename"] = f"{name} ({module.params['host']})"
            for k in obj.keys():
                if module.params[k] is True:
                    module.params[k] = "1"
                elif module.params[k] is False:
                    module.params[k] = "0"
                obj[k] = module.params[k]

            diff = obj.diff()
            if exists:
                for k in obj.keys():
                    if obj.hasChanged(k):
                        changed = True
            else:
                changed = True
            if not module.check_mode:
                if not exists:
                    obj.create()
                elif changed:
                    obj.modify()
        except Exception as err:
            module.fail_json(msg=f"Creating/editing share {name} in {container} failed: {err}")

    if state == "absent" and exists:
        try:
            obj = umc_module_for_edit("shares/share", dn)
            if not module.check_mode:
                obj.remove()
            changed = True
        except Exception as err:
            module.fail_json(msg=f"Removing share {name} in {container} failed: {err}")

    module.exit_json(changed=changed, name=name, diff=diff, container=container)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2016, Dag Wieers (@dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: sefcontext
short_description: Manages SELinux file context mapping definitions
description:
  - Manages SELinux file context mapping definitions.
  - Similar to the C(semanage fcontext) command.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.platform
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  platform:
    platforms: linux
options:
  target:
    description:
      - Target path (expression).
    type: str
    required: true
    aliases: [path]
  ftype:
    description:
      - The file type that should have SELinux contexts applied.
      - 'The following file type options are available:'
      - V(a) for all files,
      - V(b) for block devices,
      - V(c) for character devices,
      - V(d) for directories,
      - V(f) for regular files,
      - V(l) for symbolic links,
      - V(p) for named pipes,
      - V(s) for socket files.
    type: str
    choices: [a, b, c, d, f, l, p, s]
    default: a
  setype:
    description:
      - SELinux type for the specified O(target).
    type: str
  substitute:
    description:
      - Path to use to substitute file context(s) for the specified O(target). The context labeling for the O(target) subtree
        is made equivalent to this path.
      - This is also referred to as SELinux file context equivalence and it implements the C(equal) functionality of the SELinux
        management tools.
    version_added: 6.4.0
    type: str
    aliases: [equal]
  seuser:
    description:
      - SELinux user for the specified O(target).
      - Defaults to V(system_u) for new file contexts and to existing value when modifying file contexts.
    type: str
  selevel:
    description:
      - SELinux range for the specified O(target).
      - Defaults to V(s0) for new file contexts and to existing value when modifying file contexts.
    type: str
    aliases: [serange]
  state:
    description:
      - Whether the SELinux file context must be V(absent) or V(present).
      - Specifying V(absent) without either O(setype) or O(substitute) deletes both SELinux type or path substitution mappings
        that match O(target).
    type: str
    choices: [absent, present]
    default: present
  reload:
    description:
      - Reload SELinux policy after commit.
      - Note that this does not apply SELinux file contexts to existing files.
    type: bool
    default: true
  ignore_selinux_state:
    description:
      - Useful for scenarios (chrooted environment) that you cannot get the real SELinux state.
    type: bool
    default: false
notes:
  - The changes are persistent across reboots.
  - O(setype) and O(substitute) are mutually exclusive.
  - If O(state=present) then one of O(setype) or O(substitute) is mandatory.
  - The M(community.general.sefcontext) module does not modify existing files to the new SELinux context(s), so it is advisable
    to first create the SELinux file contexts before creating files, or run C(restorecon) manually for the existing files
    that require the new SELinux file contexts.
  - Not applying SELinux fcontexts to existing files is a deliberate decision as it would be unclear what reported changes
    would entail to, and there is no guarantee that applying SELinux fcontext does not pick up other unrelated prior changes.
requirements:
  - libselinux-python
  - policycoreutils-python
author:
  - Dag Wieers (@dagwieers)
"""

EXAMPLES = r"""
- name: Allow apache to modify files in /srv/git_repos
  community.general.sefcontext:
    target: '/srv/git_repos(/.*)?'
    setype: httpd_sys_rw_content_t
    state: present

- name: Substitute file contexts for path /srv/containers with /var/lib/containers
  community.general.sefcontext:
    target: /srv/containers
    substitute: /var/lib/containers
    state: present

- name: Delete file context path substitution for /srv/containers
  community.general.sefcontext:
    target: /srv/containers
    substitute: /var/lib/containers
    state: absent

- name: Delete any file context mappings for path /srv/git
  community.general.sefcontext:
    target: /srv/git
    state: absent

- name: Apply new SELinux file context to filesystem
  ansible.builtin.command: restorecon -irv /srv/git_repos
"""

RETURN = r"""
# Default return values
"""

import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib

SELINUX_IMP_ERR = None
try:
    import selinux

    HAVE_SELINUX = True
except ImportError:
    SELINUX_IMP_ERR = traceback.format_exc()
    HAVE_SELINUX = False

SEOBJECT_IMP_ERR = None
try:
    import seobject

    HAVE_SEOBJECT = True
except ImportError:
    SEOBJECT_IMP_ERR = traceback.format_exc()
    HAVE_SEOBJECT = False

# Add missing entries (backward compatible)
if HAVE_SEOBJECT:
    seobject.file_types.update(
        a=seobject.SEMANAGE_FCONTEXT_ALL,
        b=seobject.SEMANAGE_FCONTEXT_BLOCK,
        c=seobject.SEMANAGE_FCONTEXT_CHAR,
        d=seobject.SEMANAGE_FCONTEXT_DIR,
        f=seobject.SEMANAGE_FCONTEXT_REG,
        l=seobject.SEMANAGE_FCONTEXT_LINK,
        p=seobject.SEMANAGE_FCONTEXT_PIPE,
        s=seobject.SEMANAGE_FCONTEXT_SOCK,
    )

# Make backward compatible
option_to_file_type_str = dict(
    a="all files",
    b="block device",
    c="character device",
    d="directory",
    f="regular file",
    l="symbolic link",
    p="named pipe",
    s="socket",
)


def get_runtime_status(ignore_selinux_state=False):
    return True if ignore_selinux_state is True else selinux.is_selinux_enabled()


def semanage_fcontext_exists(sefcontext, target, ftype):
    """Get the SELinux file context mapping definition from policy. Return None if it does not exist."""

    # Beware that records comprise of a string representation of the file_type
    record = (target, option_to_file_type_str[ftype])
    records = sefcontext.get_all()
    try:
        return records[record]
    except KeyError:
        return None


def semanage_fcontext_substitute_exists(sefcontext, target):
    """Get the SELinux file context path substitution definition from policy. Return None if it does not exist."""

    return sefcontext.equiv_dist.get(target, sefcontext.equiv.get(target))


def semanage_fcontext_modify(module, result, target, ftype, setype, substitute, do_reload, serange, seuser, sestore=""):
    """Add or modify SELinux file context mapping definition to the policy."""

    changed = False
    prepared_diff = ""

    try:
        sefcontext = seobject.fcontextRecords(sestore)
        sefcontext.set_reload(do_reload)
        if substitute is None:
            exists = semanage_fcontext_exists(sefcontext, target, ftype)
            if exists:
                # Modify existing entry
                orig_seuser, orig_serole, orig_setype, orig_serange = exists

                if seuser is None:
                    seuser = orig_seuser
                if serange is None:
                    serange = orig_serange

                if setype != orig_setype or seuser != orig_seuser or serange != orig_serange:
                    if not module.check_mode:
                        sefcontext.modify(target, setype, ftype, serange, seuser)
                    changed = True

                    if module._diff:
                        prepared_diff += "# Change to semanage file context mappings\n"
                        prepared_diff += (
                            f"-{target}      {ftype}      {orig_seuser}:{orig_serole}:{orig_setype}:{orig_serange}\n"
                        )
                        prepared_diff += f"+{target}      {ftype}      {seuser}:{orig_serole}:{setype}:{serange}\n"
            else:
                # Add missing entry
                if seuser is None:
                    seuser = "system_u"
                if serange is None:
                    serange = "s0"

                if not module.check_mode:
                    sefcontext.add(target, setype, ftype, serange, seuser)
                changed = True

                if module._diff:
                    prepared_diff += "# Addition to semanage file context mappings\n"
                    prepared_diff += f"+{target}      {ftype}      {seuser}:object_r:{setype}:{serange}\n"
        else:
            exists = semanage_fcontext_substitute_exists(sefcontext, target)
            if exists:
                # Modify existing path substitution entry
                orig_substitute = exists

                if substitute != orig_substitute:
                    if not module.check_mode:
                        sefcontext.modify_equal(target, substitute)
                    changed = True

                    if module._diff:
                        prepared_diff += "# Change to semanage file context path substitutions\n"
                        prepared_diff += f"-{target} = {orig_substitute}\n"
                        prepared_diff += f"+{target} = {substitute}\n"
            else:
                # Add missing path substitution entry
                if not module.check_mode:
                    sefcontext.add_equal(target, substitute)
                changed = True
                if module._diff:
                    prepared_diff += "# Addition to semanage file context path substitutions\n"
                    prepared_diff += f"+{target} = {substitute}\n"

    except Exception as e:
        module.fail_json(msg=f"{e.__class__.__name__}: {e}\n")

    if module._diff and prepared_diff:
        result["diff"] = dict(prepared=prepared_diff)

    if changed and not module.check_mode:
        # Flush the in-process matchpathcon cache
        selinux.matchpathcon_fini()

    module.exit_json(changed=changed, seuser=seuser, serange=serange, **result)


def semanage_fcontext_delete(module, result, target, ftype, setype, substitute, do_reload, sestore=""):
    """Delete SELinux file context mapping definition from the policy."""

    changed = False
    prepared_diff = ""

    try:
        sefcontext = seobject.fcontextRecords(sestore)
        sefcontext.set_reload(do_reload)
        exists = semanage_fcontext_exists(sefcontext, target, ftype)
        substitute_exists = semanage_fcontext_substitute_exists(sefcontext, target)
        if exists and substitute is None:
            # Remove existing entry
            orig_seuser, orig_serole, orig_setype, orig_serange = exists

            if not module.check_mode:
                sefcontext.delete(target, ftype)
            changed = True

            if module._diff:
                prepared_diff += "# Deletion to semanage file context mappings\n"
                prepared_diff += f"-{target}      {ftype}      {exists[0]}:{exists[1]}:{exists[2]}:{exists[3]}\n"
        if (
            substitute_exists
            and setype is None
            and ((substitute is not None and substitute_exists == substitute) or substitute is None)
        ):
            # Remove existing path substitution entry
            orig_substitute = substitute_exists

            if not module.check_mode:
                sefcontext.delete(target, orig_substitute)
            changed = True

            if module._diff:
                prepared_diff += "# Deletion to semanage file context path substitutions\n"
                prepared_diff += f"-{target} = {orig_substitute}\n"

    except Exception as e:
        module.fail_json(msg=f"{e.__class__.__name__}: {e}\n")

    if module._diff and prepared_diff:
        result["diff"] = dict(prepared=prepared_diff)

    if changed and not module.check_mode:
        # Flush the in-process matchpathcon cache
        selinux.matchpathcon_fini()

    module.exit_json(changed=changed, **result)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            ignore_selinux_state=dict(type="bool", default=False),
            target=dict(type="str", required=True, aliases=["path"]),
            ftype=dict(type="str", default="a", choices=list(option_to_file_type_str.keys())),
            setype=dict(type="str"),
            substitute=dict(type="str", aliases=["equal"]),
            seuser=dict(type="str"),
            selevel=dict(type="str", aliases=["serange"]),
            state=dict(type="str", default="present", choices=["absent", "present"]),
            reload=dict(type="bool", default=True),
        ),
        mutually_exclusive=[
            ("setype", "substitute"),
            ("substitute", "ftype"),
            ("substitute", "seuser"),
            ("substitute", "selevel"),
        ],
        required_if=[
            ("state", "present", ("setype", "substitute"), True),
        ],
        supports_check_mode=True,
    )
    if not HAVE_SELINUX:
        module.fail_json(msg=missing_required_lib("libselinux-python"), exception=SELINUX_IMP_ERR)

    if not HAVE_SEOBJECT:
        module.fail_json(msg=missing_required_lib("policycoreutils-python"), exception=SEOBJECT_IMP_ERR)

    ignore_selinux_state = module.params["ignore_selinux_state"]

    if not get_runtime_status(ignore_selinux_state):
        module.fail_json(msg="SELinux is disabled on this host.")

    target = module.params["target"]
    ftype = module.params["ftype"]
    setype = module.params["setype"]
    substitute = module.params["substitute"]
    seuser = module.params["seuser"]
    serange = module.params["selevel"]
    state = module.params["state"]
    do_reload = module.params["reload"]

    result = dict(target=target, ftype=ftype, setype=setype, substitute=substitute, state=state)

    if state == "present":
        semanage_fcontext_modify(module, result, target, ftype, setype, substitute, do_reload, serange, seuser)
    elif state == "absent":
        semanage_fcontext_delete(module, result, target, ftype, setype, substitute, do_reload)
    else:
        module.fail_json(msg=f'Invalid value of argument "state": {state}')


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2021, Alexei Znamensky (russoz) <russoz@gmail.com>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: snap_alias
short_description: Manages snap aliases
version_added: 4.0.0
description:
  - Manages snaps aliases.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
options:
  state:
    description:
      - Desired state of the alias.
    type: str
    choices: [absent, present]
    default: present
  name:
    description:
      - Name of the snap.
    type: str
  alias:
    description:
      - Aliases to be created or removed.
    type: list
    elements: str
    aliases: [aliases]

author:
  - Alexei Znamensky (@russoz) <russoz@gmail.com>

seealso:
  - module: community.general.snap
"""

EXAMPLES = r"""
# Install "foo" and "bar" snap
- name: Create snap alias
  community.general.snap_alias:
    name: hello-world
    alias: hw

- name: Create multiple aliases
  community.general.snap_alias:
    name: hello-world
    aliases:
      - hw
      - hw2
      - hw3
    state: present # optional

- name: Remove one specific aliases
  community.general.snap_alias:
    name: hw
    state: absent

- name: Remove all aliases for snap
  community.general.snap_alias:
    name: hello-world
    state: absent
"""

RETURN = r"""
snap_aliases:
  description: The snap aliases after execution. If called in check mode, then the list represents the state before execution.
  type: list
  elements: str
  returned: always
version:
  description: Versions of snap components as reported by C(snap version).
  type: dict
  returned: always
  version_added: 10.3.0
"""


import re

from ansible_collections.community.general.plugins.module_utils._module_helper import StateModuleHelper
from ansible_collections.community.general.plugins.module_utils._snap import get_version, snap_runner


class SnapAlias(StateModuleHelper):
    _RE_ALIAS_LIST = re.compile(r"^(?P<snap>\S+)\s+(?P<alias>[\w-]+)\s+.*$")

    module = dict(
        argument_spec={
            "state": dict(type="str", choices=["absent", "present"], default="present"),
            "name": dict(type="str"),
            "alias": dict(type="list", elements="str", aliases=["aliases"]),
        },
        required_if=[
            ("state", "present", ["name", "alias"]),
            ("state", "absent", ["name", "alias"], True),
        ],
        supports_check_mode=True,
    )

    def _aliases(self):
        n = self.vars.name
        return {n: self._get_aliases_for(n)} if n else self._get_aliases()

    def __init_module__(self):
        self.runner = snap_runner(self.module)
        self.vars.version = get_version(self.runner)
        self.vars.set("snap_aliases", self._aliases(), change=True, diff=True)

    def __quit_module__(self):
        self.vars.snap_aliases = self._aliases()

    def _get_aliases(self):
        def process(rc, out, err):
            if err:
                return {}
            aliases = [self._RE_ALIAS_LIST.match(a.strip()) for a in out.splitlines()[1:]]
            snap_alias_list = [(entry.group("snap"), entry.group("alias")) for entry in aliases]
            results = {}
            for snap, alias in snap_alias_list:
                results[snap] = results.get(snap, []) + [alias]
            return results

        with self.runner("state_alias name", check_rc=True, output_process=process) as ctx:
            aliases = ctx.run(state_alias="info")
            if self.verbosity >= 4:
                self.vars.get_aliases_run_info = ctx.run_info
            return aliases

    def _get_aliases_for(self, name):
        return self._get_aliases().get(name, [])

    def _has_alias(self, name=None, alias=None):
        if name:
            if name not in self.vars.snap_aliases:
                return False
            if alias is None:
                return bool(self.vars.snap_aliases[name])
            return alias in self.vars.snap_aliases[name]

        return any(alias in aliases for aliases in self.vars.snap_aliases.values())

    def state_present(self):
        for _alias in self.vars.alias:
            if not self._has_alias(self.vars.name, _alias):
                self.changed = True
                with self.runner("state_alias name alias", check_mode_skip=True) as ctx:
                    ctx.run(state_alias=self.vars.state, alias=_alias)
                    if self.verbosity >= 4:
                        self.vars.run_info = ctx.run_info

    def state_absent(self):
        if not self.vars.alias:
            if self._has_alias(self.vars.name):
                self.changed = True
                with self.runner("state_alias name", check_mode_skip=True) as ctx:
                    ctx.run(state_alias=self.vars.state)
                    if self.verbosity >= 4:
                        self.vars.run_info = ctx.run_info
        else:
            for _alias in self.vars.alias:
                if self._has_alias(self.vars.name, _alias):
                    self.changed = True
                    with self.runner("state_alias alias", check_mode_skip=True) as ctx:
                        ctx.run(state_alias=self.vars.state, alias=_alias)
                        if self.verbosity >= 4:
                            self.vars.run_info = ctx.run_info


def main():
    SnapAlias.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2015, Björn Andersson
# Copyright (c) 2021, Ansible Project
# Copyright (c) 2021, Abhijeet Kasurde <akasurde@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ssh_config
short_description: Manage SSH config for user
version_added: '2.0.0'
description:
  - Configures SSH hosts with special C(IdentityFile)s and hostnames.
author:
  - Björn Andersson (@gaqzi)
  - Abhijeet Kasurde (@Akasurde)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  state:
    description:
      - Whether a host entry should exist or not.
    default: present
    choices: ['present', 'absent']
    type: str
  user:
    description:
      - Which user account this configuration file belongs to.
      - If none given and O(ssh_config_file) is not specified, C(/etc/ssh/ssh_config) is used.
      - If a user is given, C(~/.ssh/config) is used.
      - Mutually exclusive with O(ssh_config_file).
    type: str
  group:
    description:
      - Which group this configuration file belongs to.
      - If none given, O(user) is used.
    type: str
  host:
    description:
      - The endpoint this configuration is valid for.
      - It can be an actual address on the internet or an alias that connects to the value of O(hostname).
    required: true
    type: str
  hostname:
    description:
      - The actual host to connect to when connecting to the host defined.
    type: str
  port:
    description:
      - The actual port to connect to when connecting to the host defined.
    type: str
  remote_user:
    description:
      - Specifies the user to log in as.
    type: str
  identity_file:
    description:
      - The path to an identity file (SSH private key) that is used when connecting to this host.
      - File need to exist and have mode V(0600) to be valid.
    type: path
  identities_only:
    description:
      - Specifies that SSH should only use the configured authentication identity and certificate files (either the default
        files, or those explicitly configured in the C(ssh_config) files or passed on the ssh command-line), even if C(ssh-agent)
        or a C(PKCS11Provider) or C(SecurityKeyProvider) offers more identities.
    type: bool
    version_added: 8.2.0
  user_known_hosts_file:
    description:
      - Sets the user known hosts file option.
    type: str
  strict_host_key_checking:
    description:
      - Whether to strictly check the host key when doing connections to the remote host.
      - The value V(accept-new) is supported since community.general 8.6.0.
    choices: ['yes', 'no', 'ask', 'accept-new']
    type: str
  proxycommand:
    description:
      - Sets the C(ProxyCommand) option.
      - Mutually exclusive with O(proxyjump).
    type: str
  proxyjump:
    description:
      - Sets the C(ProxyJump) option.
      - Mutually exclusive with O(proxycommand).
    type: str
    version_added: 6.5.0
  forward_agent:
    description:
      - Sets the C(ForwardAgent) option.
    type: bool
    version_added: 4.0.0
  add_keys_to_agent:
    description:
      - Sets the C(AddKeysToAgent) option.
    type: bool
    version_added: 8.2.0
  ssh_config_file:
    description:
      - SSH config file.
      - If O(user) and this option are not specified, C(/etc/ssh/ssh_config) is used.
      - Mutually exclusive with O(user).
    type: path
  host_key_algorithms:
    description:
      - Sets the C(HostKeyAlgorithms) option.
    type: str
    version_added: 6.1.0
  controlmaster:
    description:
      - Sets the C(ControlMaster) option.
    choices: ['yes', 'no', 'ask', 'auto', 'autoask']
    type: str
    version_added: 8.1.0
  controlpath:
    description:
      - Sets the C(ControlPath) option.
    type: str
    version_added: 8.1.0
  controlpersist:
    description:
      - Sets the C(ControlPersist) option.
    type: str
    version_added: 8.1.0
  dynamicforward:
    description:
      - Sets the C(DynamicForward) option.
    type: str
    version_added: 10.1.0
  other_options:
    description:
      - Allows specifying arbitrary SSH config entry options using a dictionary.
      - The key names must be lower case. Keys with upper case values are rejected.
      - The values must be strings. Other values are rejected.
    type: dict
    version_added: 10.4.0
  address_family:
    description:
      - Sets the C(AddressFamily) option.
    choices: ['any', 'inet', 'inet6']
    type: str
    version_added: 13.0.0
requirements:
  - paramiko
"""

EXAMPLES = r"""
- name: Add a host in the configuration
  community.general.ssh_config:
    user: akasurde
    host: "example.com"
    hostname: "github.com"
    identity_file: "/home/akasurde/.ssh/id_rsa"
    port: '2223'
    state: present
    other_options:
      serveraliveinterval: '30'

- name: Add SSH config with key auto-added to agent
  community.general.ssh_config:
    user: devops
    host: "example.com"
    hostname: "staging.example.com"
    identity_file: "/home/devops/.ssh/id_rsa"
    add_keys_to_agent: true
    state: present

- name: Delete a host from the configuration
  community.general.ssh_config:
    ssh_config_file: "{{ ssh_config_test }}"
    host: "example.com"
    state: absent
"""

RETURN = r"""
hosts_added:
  description: A list of host added.
  returned: success
  type: list
  sample: ["example.com"]
hosts_removed:
  description: A list of host removed.
  returned: success
  type: list
  sample: ["example.com"]
hosts_changed:
  description: A list of host changed.
  returned: success
  type: list
  sample: ["example.com"]
hosts_change_diff:
  description: A list of host diff changes.
  returned: on change
  type: list
  sample:
    [
      {
        "example.com": {
          "new": {
            "hostname": "github.com",
            "identityfile": [
              "/tmp/test_ssh_config/fake_id_rsa"
            ],
            "port": "2224"
          },
          "old": {
            "hostname": "github.com",
            "identityfile": [
              "/tmp/test_ssh_config/fake_id_rsa"
            ],
            "port": "2224"
          }
        }
      }
    ]
"""

import os
from copy import deepcopy

from ansible.module_utils.basic import AnsibleModule, missing_required_lib

from ansible_collections.community.general.plugins.module_utils._ssh import determine_config_file
from ansible_collections.community.general.plugins.module_utils._stormssh import (
    HAS_PARAMIKO,
    PARAMIKO_IMPORT_ERROR,
    ConfigParser,
)


def convert_bool(value):
    if value is True:
        return "yes"
    if value is False:
        return "no"
    return None


def fix_bool_str(value):
    if value == "True":
        return "yes"
    if value == "False":
        return "no"
    return value


class SSHConfig:
    def __init__(self, module):
        self.module = module
        if not HAS_PARAMIKO:
            module.fail_json(msg=missing_required_lib("PARAMIKO"), exception=PARAMIKO_IMPORT_ERROR)
        self.params = module.params
        self.user = self.params.get("user")
        self.group = self.params.get("group") or self.user
        self.host = self.params.get("host")
        self.config_file = self.params.get("ssh_config_file")
        self.identity_file = self.params["identity_file"]
        self.check_ssh_config_path()
        try:
            self.config = ConfigParser(self.config_file)
        except FileNotFoundError:
            self.module.fail_json(msg=f"Failed to find {self.config_file}")
        self.config.load()

    def check_ssh_config_path(self):
        self.config_file = determine_config_file(self.user, self.config_file)

        # See if the identity file exists or not, relative to the config file
        if os.path.exists(self.config_file) and self.identity_file is not None:
            dirname = os.path.dirname(self.config_file)
            self.identity_file = os.path.join(dirname, self.identity_file)

            if not os.path.exists(self.identity_file):
                self.module.fail_json(msg=f"IdentityFile {self.params['identity_file']} does not exist")

    def ensure_state(self):
        hosts_result = self.config.search_host(self.host)
        state = self.params["state"]
        args = dict(
            hostname=self.params.get("hostname"),
            port=self.params.get("port"),
            identity_file=self.params.get("identity_file"),
            identities_only=convert_bool(self.params.get("identities_only")),
            user=self.params.get("remote_user"),
            strict_host_key_checking=self.params.get("strict_host_key_checking"),
            user_known_hosts_file=self.params.get("user_known_hosts_file"),
            proxycommand=self.params.get("proxycommand"),
            proxyjump=self.params.get("proxyjump"),
            host_key_algorithms=self.params.get("host_key_algorithms"),
            forward_agent=convert_bool(self.params.get("forward_agent")),
            add_keys_to_agent=convert_bool(self.params.get("add_keys_to_agent")),
            controlmaster=self.params.get("controlmaster"),
            controlpath=self.params.get("controlpath"),
            controlpersist=fix_bool_str(self.params.get("controlpersist")),
            dynamicforward=self.params.get("dynamicforward"),
            address_family=self.params.get("address_family"),
        )
        if self.params.get("other_options"):
            for key, value in self.params.get("other_options").items():
                if key.lower() != key:
                    self.module.fail_json(msg=f"The other_options key {key} must be lower case")
                if key not in args:
                    if not isinstance(value, str):
                        self.module.fail_json(
                            msg=f"The other_options value provided for key {key} must be a string, got {type(value)}"
                        )
                    args[key] = value
                else:
                    self.module.fail_json(msg=f"Multiple values provided for key {key}")

        config_changed = False
        hosts_changed = []
        hosts_change_diff = []
        hosts_removed = []
        hosts_added = []

        hosts_result = [host for host in hosts_result if host["host"] == self.host]

        if hosts_result:
            for host in hosts_result:
                if state == "absent":
                    # Delete host from the configuration
                    config_changed = True
                    hosts_removed.append(host["host"])
                    self.config.delete_host(host["host"])
                else:
                    # Update host in the configuration
                    changed, options = self.change_host(host["options"], **args)

                    if changed:
                        config_changed = True
                        self.config.update_host(host["host"], options)
                        hosts_changed.append(host["host"])
                        hosts_change_diff.append(
                            {
                                host["host"]: {
                                    "old": host["options"],
                                    "new": options,
                                }
                            }
                        )
        elif state == "present":
            changed, options = self.change_host(dict(), **args)

            if changed:
                config_changed = True
                hosts_added.append(self.host)
                self.config.add_host(self.host, options)

        if config_changed and not self.module.check_mode:
            try:
                self.config.write_to_ssh_config()
            except PermissionError as perm_exec:
                self.module.fail_json(msg=f"Failed to write to {self.config_file} due to permission issue: {perm_exec}")
            # Make sure we set the permission
            perm_mode = "0600"
            if self.config_file == "/etc/ssh/ssh_config":
                perm_mode = "0644"
            self.module.set_mode_if_different(self.config_file, perm_mode, False)
            # Make sure the file is owned by the right user and group
            self.module.set_owner_if_different(self.config_file, self.user, False)
            self.module.set_group_if_different(self.config_file, self.group, False)

        self.module.exit_json(
            changed=config_changed,
            hosts_changed=hosts_changed,
            hosts_removed=hosts_removed,
            hosts_change_diff=hosts_change_diff,
            hosts_added=hosts_added,
        )

    @staticmethod
    def change_host(options, **kwargs):
        options = deepcopy(options)
        changed = False
        for k, v in kwargs.items():
            if "_" in k:
                k = k.replace("_", "")

            if not v:
                if options.get(k):
                    del options[k]
                    changed = True
            elif options.get(k) != v and not (isinstance(options.get(k), list) and v in options.get(k)):
                options[k] = v
                changed = True

        return changed, options


def main():
    module = AnsibleModule(
        argument_spec=dict(
            group=dict(type="str"),
            host=dict(type="str", required=True),
            hostname=dict(type="str"),
            host_key_algorithms=dict(type="str", no_log=False),
            identity_file=dict(type="path"),
            identities_only=dict(type="bool"),
            other_options=dict(type="dict"),
            port=dict(type="str"),
            proxycommand=dict(type="str"),
            proxyjump=dict(type="str"),
            forward_agent=dict(type="bool"),
            add_keys_to_agent=dict(type="bool"),
            remote_user=dict(type="str"),
            ssh_config_file=dict(type="path"),
            state=dict(type="str", default="present", choices=["present", "absent"]),
            strict_host_key_checking=dict(type="str", choices=["yes", "no", "ask", "accept-new"]),
            controlmaster=dict(type="str", choices=["yes", "no", "ask", "auto", "autoask"]),
            controlpath=dict(type="str"),
            controlpersist=dict(type="str"),
            dynamicforward=dict(type="str"),
            user=dict(type="str"),
            user_known_hosts_file=dict(type="str"),
            address_family=dict(type="str", choices=["any", "inet", "inet6"]),
        ),
        supports_check_mode=True,
        mutually_exclusive=[
            ["user", "ssh_config_file"],
            ["proxycommand", "proxyjump"],
        ],
    )

    ssh_config_obj = SSHConfig(module)
    ssh_config_obj.ensure_state()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2016, Mathieu Bultel <mbultel@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pacemaker_cluster
short_description: Manage pacemaker clusters
author:
  - Mathieu Bultel (@matbu)
  - Dexter Le (@munchtoast)
description:
  - This module can manage a pacemaker cluster and nodes from Ansible using the pacemaker CLI.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  state:
    description:
      - Indicate desired state of the cluster.
      - The value V(maintenance) has been added in community.general 11.1.0.
    choices: [cleanup, offline, online, restart, maintenance]
    type: str
    required: true
  name:
    description:
      - Specify which node of the cluster you want to manage. V(null) == the cluster status itself, V(all) == check the status
        of all nodes.
    type: str
    aliases: ['node']
  timeout:
    description:
      - Timeout period (in seconds) for polling the cluster operation.
    type: int
    default: 300
  force:
    description:
      - Force the change of the cluster state.
    type: bool
    default: true
"""

EXAMPLES = r"""
- name: Set cluster Online
  hosts: localhost
  gather_facts: false
  tasks:
    - name: Get cluster state
      community.general.pacemaker_cluster:
        state: online
"""

RETURN = r"""
out:
  description: The output of the current state of the cluster. It returns a list of the nodes state.
  type: str
  sample: 'out: [["  overcloud-controller-0", " Online"]]}'
  returned: always
"""

from ansible_collections.community.general.plugins.module_utils._module_helper import StateModuleHelper
from ansible_collections.community.general.plugins.module_utils._pacemaker import (
    get_pacemaker_maintenance_mode,
    pacemaker_runner,
)


class PacemakerCluster(StateModuleHelper):
    module = dict(
        argument_spec=dict(
            state=dict(type="str", choices=["cleanup", "offline", "online", "restart", "maintenance"], required=True),
            name=dict(type="str", aliases=["node"]),
            timeout=dict(type="int", default=300),
            force=dict(type="bool", default=True),
        ),
        supports_check_mode=True,
    )
    default_state = ""

    def __init_module__(self):
        self.runner = pacemaker_runner(self.module)
        self.vars.set("apply_all", True if not self.module.params["name"] else False)
        get_args = dict(cli_action="cluster", state="status", name=None, apply_all=self.vars.apply_all)
        if self.module.params["state"] == "maintenance":
            get_args["cli_action"] = "property"
            get_args["state"] = "config"
            get_args["name"] = "maintenance-mode"
        elif self.module.params["state"] == "cleanup":
            get_args["cli_action"] = "resource"
            get_args["name"] = self.module.params["name"]

        self.vars.set("get_args", get_args)
        self.vars.set("previous_value", self._get()["out"])
        self.vars.set("value", self.vars.previous_value, change=True, diff=True)

        if self.module.params["state"] == "cleanup":
            self.module.deprecate(
                'The value `cleanup` for "state" is being deprecated, use pacemaker_resource module instead.',
                version="14.0.0",
                collection_name="community.general",
            )

    def __quit_module__(self):
        self.vars.set("value", self._get()["out"])

    def _process_command_output(self, fail_on_err, ignore_err_msg=""):
        def process(rc, out, err):
            if fail_on_err and rc != 0 and err and ignore_err_msg not in err:
                self.do_raise(f"pcs failed with error (rc={rc}): {err}")
            out = out.rstrip()
            return None if out == "" else out

        return process

    def _get(self):
        with self.runner("cli_action state name") as ctx:
            result = ctx.run(
                cli_action=self.vars.get_args["cli_action"],
                state=self.vars.get_args["state"],
                name=self.vars.get_args["name"],
            )
            return dict(rc=result[0], out=(result[1] if result[1] != "" else None), err=result[2])

    def state_cleanup(self):
        with self.runner(
            "cli_action state name", output_process=self._process_command_output(True, "Fail"), check_mode_skip=True
        ) as ctx:
            ctx.run(cli_action="resource")

    def state_offline(self):
        with self.runner(
            "cli_action state name apply_all wait",
            output_process=self._process_command_output(True, "not currently running"),
            check_mode_skip=True,
        ) as ctx:
            ctx.run(cli_action="cluster", apply_all=self.vars.apply_all, wait=self.module.params["timeout"])

    def state_online(self):
        with self.runner(
            "cli_action state name apply_all wait",
            output_process=self._process_command_output(True, "currently running"),
            check_mode_skip=True,
        ) as ctx:
            ctx.run(cli_action="cluster", apply_all=self.vars.apply_all, wait=self.module.params["timeout"])

        if get_pacemaker_maintenance_mode(self.runner):
            with self.runner(
                "cli_action state name", output_process=self._process_command_output(True, "Fail"), check_mode_skip=True
            ) as ctx:
                ctx.run(cli_action="property", state="maintenance", name="maintenance-mode=false")

    def state_maintenance(self):
        with self.runner(
            "cli_action state name", output_process=self._process_command_output(True, "Fail"), check_mode_skip=True
        ) as ctx:
            ctx.run(cli_action="property", name="maintenance-mode=true")

    def state_restart(self):
        with self.runner(
            "cli_action state name apply_all wait",
            output_process=self._process_command_output(True, "not currently running"),
            check_mode_skip=True,
        ) as ctx:
            ctx.run(
                cli_action="cluster", state="offline", apply_all=self.vars.apply_all, wait=self.module.params["timeout"]
            )
            ctx.run(
                cli_action="cluster", state="online", apply_all=self.vars.apply_all, wait=self.module.params["timeout"]
            )

        if get_pacemaker_maintenance_mode(self.runner):
            with self.runner(
                "cli_action state name", output_process=self._process_command_output(True, "Fail"), check_mode_skip=True
            ) as ctx:
                ctx.run(cli_action="property", state="maintenance", name="maintenance-mode=false")


def main():
    PacemakerCluster.execute()


if __name__ == "__main__":
    main()
